<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2017-2021 XMLmind Software. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind DITA Converter project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns="http://www.w3.org/1999/xhtml"
                xmlns:ditac="http://www.xmlmind.com/ditac/schema/ditac"
                xmlns:u="http://www.xmlmind.com/namespace/ditac"
                exclude-result-prefixes="xs ditac u"
                version="2.0">

  <!-- commonAttributes ================================================== -->

  <xsl:template name="commonAttributes">
    <xsl:param name="class" select="''"/>
    <xsl:param name="classPrefix" select="''"/>
    <xsl:param name="extraClasses" select="''"/>
    
    <xsl:call-template name="idAttribute"/>
    <xsl:call-template name="localizationAttributes"/>
    <xsl:call-template name="titleAttribute"/>
    <xsl:call-template name="otherAttributes">
      <xsl:with-param name="class" select="$class"/>
      <xsl:with-param name="classPrefix" select="$classPrefix"/>
      <xsl:with-param name="extraClasses" select="$extraClasses"/>
    </xsl:call-template>
    <xsl:call-template name="flagsAttribute"/>
  </xsl:template>

  <xsl:template name="commonAttributes2">
    <xsl:param name="class" select="''"/>
    <xsl:param name="classPrefix" select="''"/>
    <xsl:param name="extraClasses" select="''"/>
    
    <xsl:call-template name="idAttribute2"/>
    <xsl:call-template name="localizationAttributes"/>
    <xsl:call-template name="otherAttributes">
      <xsl:with-param name="class" select="$class"/>
      <xsl:with-param name="classPrefix" select="$classPrefix"/>
      <xsl:with-param name="extraClasses" select="$extraClasses"/>
    </xsl:call-template>
    <xsl:call-template name="flagsAttribute"/>
  </xsl:template>

  <xsl:template name="idAttribute">
    <xsl:copy-of select="@id"/>
  </xsl:template>

  <xsl:template name="idAttribute2">
    <xsl:attribute name="id">
      <xsl:choose>
        <xsl:when test="@id"><xsl:value-of select="@id"/></xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('I_', generate-id(), '_')"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>

  <xsl:template name="localizationAttributes">
    <xsl:copy-of select="@xml:lang"/>
    <xsl:if test="$xhtmlVersion ne '1.1' and exists(@xml:lang)">
      <xsl:attribute name="lang" select="string(@xml:lang)"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="titleAttribute">
    <xsl:if test="@ditac:title">
      <xsl:attribute name="title" select="string(@ditac:title)"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="otherAttributes">
    <xsl:param name="class" select="''"/>
    <xsl:param name="classPrefix" select="''"/>
    <xsl:param name="extraClasses" select="''"/>

    <xsl:call-template name="otherAttributesImpl">
      <xsl:with-param name="class" select="$class"/>
      <xsl:with-param name="classPrefix" select="$classPrefix"/>
      <xsl:with-param name="extraClasses" select="$extraClasses"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="otherAttributesImpl">
    <xsl:param name="class" select="''"/>
    <xsl:param name="classPrefix" select="''"/>
    <xsl:param name="extraClasses" select="''"/>

    <xsl:variable name="baseClass">
      <xsl:choose>
        <xsl:when test="$class">
          <xsl:value-of select="$class"/>
        </xsl:when>
        <xsl:when test="@class">
          <xsl:value-of 
              select="concat($classPrefix, u:classToElementName(@class))"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
 
    <xsl:variable name="classes">
      <xsl:choose>
        <xsl:when test="$extraClasses ne ''">
          <xsl:value-of select="concat($baseClass, ' ', $extraClasses)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$baseClass"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- Discard ol and table elements pseudo output classes. -->

    <xsl:variable name="outputclass">
      <xsl:choose>
        <xsl:when test="exists(@outputclass)">
          <xsl:variable name="classList">
            <xsl:sequence select="for $token in tokenize(@outputclass, '\s+')
                                  return 
                                      if (not($token eq 'lower-alpha' or 
                                              $token eq 'upper-alpha' or 
                                              $token eq 'lower-roman' or 
                                              $token eq 'upper-roman' or 
                                              $token eq 'decimal' or 
                                              $token eq 'continue' or 
                                              $token eq 'inheritnum' or 
                                              starts-with($token, 'start(') or
                                              starts-with($token, 'bgcolor(')))
                                      then $token
                                      else ()" />
          </xsl:variable>
          <xsl:value-of select="string-join($classList, ' ')"/>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="@outputclass"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="classValue">
      <xsl:choose>
        <!-- JavaHelp does not support multiple classes. -->
        <xsl:when test="$xhtmlVersion eq '-3.2'">
          <xsl:value-of select="$baseClass"/>
        </xsl:when>

        <xsl:when test="$outputclass ne ''">
          <xsl:value-of select="concat($classes, ' ', $outputclass)"/>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of  select="$classes"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$classValue ne ''">
      <xsl:attribute name="class" select="$classValue"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="namedAnchor">
    <!-- This is a hack created uniquely to support JavaHelp which cannot
         scroll to id="XXX" and instead insists on having a name="XXX". 
         Note that the HTML 3.2 created by these XSLT stylesheets 
         cannot be valid in all cases. -->
    <xsl:if test="exists(@id) and $xhtmlVersion eq '-3.2'">
      <a name="{string(@id)}"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="namedAnchor2">
    <xsl:if test="$xhtmlVersion eq '-3.2'">
      <xsl:variable name="id">
        <xsl:choose>
          <xsl:when test="@id"><xsl:value-of select="@id"/></xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat('I_', generate-id(), '_')"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <a name="{$id}"/>
    </xsl:if>
  </xsl:template>

  <!-- flagsAttribute ==================================================== -->

  <xsl:template name="flagsAttribute">
    <xsl:variable name="style" select="u:flagsAttributeValue(.)"/>
    <xsl:if test="$style ne ''">
      <xsl:attribute name="style" select="$style"/>
    </xsl:if>
  </xsl:template>

  <xsl:function name="u:flagsAttributeValue" as="xs:string">
    <xsl:param name="context" as="element()"/>

    <xsl:variable name="props">
      <xsl:for-each select="($context/@ditac:flags-color, 
                             $context/@ditac:flags-background-color, 
                             $context/@ditac:flags-font-weight,
                             $context/@ditac:flags-font-style,
                             $context/@ditac:flags-text-decoration)">
        <xsl:sequence 
          select="concat(substring-after(local-name(),'flags-'),': ', ., ';')"/>
      </xsl:for-each>
    </xsl:variable>

    <xsl:sequence select="normalize-space(string-join($props, ' '))"/>
  </xsl:function>

  <!-- displayAttributes ================================================= -->

  <xsl:template name="displayAttributes">
    <!-- @expanse not implemented here -->
    <xsl:variable name="style">
      <xsl:if test="@scale">
        font-size: <xsl:value-of select="@scale"/>%;
      </xsl:if>

      <xsl:if test="@frame">
        <!-- Do not use shorthand properties and do no specify border
             color. This allows the user to specify a custom border color in
             her/his CSS. -->
        <xsl:choose>
          <xsl:when test="@frame eq 'top'">
            border-top-width: 1px;
            border-top-style: solid;
            padding-top: 0.5ex;
          </xsl:when>
          <xsl:when test="@frame eq 'bottom'">
            border-bottom-width: 1px;
            border-bottom-style: solid;
            padding-bottom: 0.5ex;
          </xsl:when>
          <xsl:when test="@frame eq 'topbot'">
            border-top-width: 1px;
            border-top-style: solid;
            border-bottom-width: 1px;
            border-bottom-style: solid;
            padding-top: 0.5ex;
            padding-bottom: 0.5ex;
          </xsl:when>
          <xsl:when test="@frame eq 'all'">
            border-width: 1px;
            border-style: solid;
            padding: 0.5ex;
          </xsl:when>
          <xsl:when test="@frame eq 'sides'">
            border-left-width: 1px; 
            border-left-style: solid; 
            border-right-width: 1px;
            border-right-style: solid;
            padding-left: 0.5ex;
            padding-right: 0.5ex;
          </xsl:when>
        </xsl:choose>
      </xsl:if>
      
      <!-- A "equation-d/equation-figure" is also a "topic/fig".
           "svg-d/svg-container" and "mathml-d/mathml" are 
           also "topic/foreign". -->
      <xsl:if test="contains(@class,' topic/fig ') and 
                (index-of($centerList, u:classToElementName(@class)) ge 1) and
                exists(./*[contains(@class,' topic/image ') or
                           contains(@class,' topic/foreign ')])">
        text-align: center;
      </xsl:if>
    </xsl:variable>

    <xsl:if test="$style ne ''">
      <!-- displayAttributes is invoked for elements which are fully flaggable.
           No need to concat the styles returned by u:flagsAttributeValue(). -->
      <xsl:attribute name="style" select="normalize-space($style)"/>
    </xsl:if>
  </xsl:template>

  <!-- External links ==================================================== -->

  <xsl:template name="externalLinkAttributes">
    <!-- No need to lookup for @scope: the preprocessor has cascaded 
         this attribute in the related-links section. -->
    <xsl:if test="@scope eq 'external'">
      <xsl:if test="$xhtmlVersion ne '1.1'">
        <xsl:attribute name="target">_blank</xsl:attribute>
      </xsl:if>

      <xsl:variable name="format"
                    select="lower-case(normalize-space(@format))" />
      <xsl:if test="$format ne ''">
        <xsl:variable name="i" select="index-of($formatToType, $format)" />
        <xsl:variable name="type"
          select="if ($i &gt; 0) then $formatToType[$i + 1] else ''" />
        <xsl:if test="$type ne ''">
          <xsl:attribute name="type" select="$type" />
        </xsl:if>
      </xsl:if>
    </xsl:if>
  </xsl:template>

  <xsl:template name="externalLinkIcon">
    <!-- No need to lookup for @scope: the preprocessor has cascaded 
         this attribute in the related-links section. -->
    <xsl:if test="@scope eq 'external'">
      <xsl:call-template name="addExternalLinkIcon"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="addExternalLinkIcon">
    <xsl:if test="$mark-external-links eq 'yes' and $xhtmlVersion ne '1.1'">
      <xsl:text> </xsl:text>
      <img>
        <xsl:call-template name="noImageBorder"/>

        <xsl:attribute name="src"
                       select="concat($xslResourcesDir,
                                      $external-link-icon-name)"/>
        <xsl:attribute name="alt">
          <xsl:call-template name="localize">
            <xsl:with-param name="message" select="'newWindow'"/>
          </xsl:call-template>
        </xsl:attribute>
        <xsl:if test="$external-link-icon-width ne ''">
          <xsl:attribute name="width" select="$external-link-icon-width"/>
        </xsl:if>
        <xsl:if test="$external-link-icon-height ne ''">
          <xsl:attribute name="height" select="$external-link-icon-height"/>
        </xsl:if>
      </img>
    </xsl:if>
  </xsl:template>

  <xsl:template name="noImageBorder">
    <xsl:choose>
      <xsl:when test="$xhtmlVersion eq '5.0' or $xhtmlVersion eq '1.1'">
        <xsl:attribute name="style" select="'border-style: none;'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="border" select="'0'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- tableLayoutAttributes ============================================= -->

  <xsl:template name="tableLayoutAttributes">
    <xsl:param name="width" select="''"/>
    <xsl:param name="cellspacing" select="''"/>
    <xsl:param name="align" select="''"/>
    <xsl:param name="valign" select="''"/>
    <xsl:param name="extraStyles" select="''"/>

    <xsl:choose>
      <xsl:when test="$xhtmlVersion eq '5.0'">
        <xsl:variable name="style">
          <xsl:if test="$width ne ''">
            <xsl:variable name="width2" 
                          select="if (number($width) gt 0) 
                                  then concat($width, 'px')
                                  else $width"/>
            <xsl:value-of select="concat('width: ', $width2, ';')"/>
          </xsl:if>

          <xsl:if test="$cellspacing ne ''">
            <xsl:value-of
              select="concat('border-spacing: ', $cellspacing, ';')"/>
          </xsl:if>

          <!-- align="char" does not seem to be supported by Web browsers. -->
          <xsl:if test="$align ne '' and $align ne 'char'">
            <xsl:value-of select="concat('text-align: ', $align, ';')"/>
          </xsl:if>

          <xsl:if test="$valign ne ''">
            <xsl:value-of select=" concat('vertical-align: ', $valign, ';')"/>
          </xsl:if>

          <xsl:value-of select="$extraStyles"/>
        </xsl:variable>
          
        <xsl:variable name="style2" select="normalize-space($style)"/>
        <xsl:if test="$style2 ne ''">
          <xsl:attribute name="style" select="$style2"/>
        </xsl:if>
      </xsl:when>

      <xsl:otherwise>
        <xsl:if test="$width ne ''">
          <xsl:attribute name="width" select="$width"/>
        </xsl:if>

        <xsl:if test="$cellspacing ne ''">
          <xsl:attribute name="cellspacing" select="$cellspacing"/>
        </xsl:if>

        <!-- align="char" does not seem to be supported by Web browsers. -->
        <xsl:if test="$align ne '' and $align ne 'char'">
          <xsl:attribute name="align" select="$align"/>
        </xsl:if>

        <xsl:if test="$valign ne ''">
          <xsl:choose>
            <xsl:when test="$xhtmlVersion eq '-3.2' and $valign eq 'baseline'">
              <xsl:attribute name="valign" select="'top'"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="valign" select="$valign"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>

        <xsl:variable name="extraStyles2" 
                      select="normalize-space($extraStyles)"/>
        <xsl:if test="$extraStyles2 ne ''">
          <xsl:attribute name="style" select="$extraStyles2"/>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- outputclass=bgcolor(XXX) support ================================== -->

  <xsl:template name="bgcolorStyle">
    <xsl:variable name="bgcolor"
      select="normalize-space(substring-before(substring-after(@outputclass, 
                                                               'bgcolor('), 
                                               ')'))"/>

    <xsl:sequence select="if ($bgcolor ne '') 
                          then concat('background-color: ', $bgcolor, ';') 
                          else ''"/>
  </xsl:template>

</xsl:stylesheet>
