<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:d="http://docbook.org/ns/docbook"
                exclude-result-prefixes="d"
                version='1.0'>

  <xsl:param name="body.font.master">11</xsl:param>
  <xsl:param name="line-height">1.2</xsl:param>
  <xsl:param name="orderedlist.label.width">1.5em</xsl:param>
  <xsl:param name="title.font.family">sans-serif</xsl:param>
  
  <!-- para, simpara, etc ========== -->
  
  <xsl:attribute-set name="normal.spacing.properties">
    <xsl:attribute name="space-before.minimum">0.8em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">1em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">1.2em</xsl:attribute>

    <!-- WHY THIS?
         RenderX XEP VERY annoying limitation:
         space-after.conditionality="discard" is not implemented,
         fallback value is "retain". -->
    
    <xsl:attribute name="space-after.minimum">
      <xsl:choose>
        <xsl:when test="$xep.extensions=1">0pt</xsl:when>
        <xsl:otherwise>0.8em</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <xsl:attribute name="space-after.optimum">
      <xsl:choose>
        <xsl:when test="$xep.extensions=1">0pt</xsl:when>
        <xsl:otherwise>1em</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <xsl:attribute name="space-after.maximum">
      <xsl:choose>
        <xsl:when test="$xep.extensions=1">0pt</xsl:when>
        <xsl:otherwise>1.2em</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="normal.para.spacing"
                     use-attribute-sets="normal.spacing.properties">
  </xsl:attribute-set>

  <!-- programlisting, etc ========== -->
  
  <xsl:attribute-set name="shade.verbatim.style">
    <xsl:attribute name="background-color">#F0F0F0</xsl:attribute>
    <xsl:attribute name="padding">0.25em 0.5em</xsl:attribute>
    <xsl:attribute name="border">0.5pt solid #E0E0E0</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="verbatim.properties"
                     use-attribute-sets="normal.spacing.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="monospace.properties">
    <xsl:attribute name="font-size">0.83em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="monospace.spacing.properties">
    <xsl:attribute name="space-before.minimum">1em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">1.2em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">1.4em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">1em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">1.2em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">1.4em</xsl:attribute>
    <xsl:attribute name="margin-{$direction.align.start}">0.125em</xsl:attribute>
    <xsl:attribute name="margin-{$direction.align.end}">0.125em</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="monospace.verbatim.properties"
                     use-attribute-sets="monospace.spacing.properties">
    <xsl:attribute name="line-height">1.44</xsl:attribute>
  </xsl:attribute-set>
  
  <!-- lists ========== -->
  
  <xsl:attribute-set name="list.block.spacing"
                     use-attribute-sets="normal.spacing.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="list.item.spacing"
                     use-attribute-sets="normal.spacing.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="no.spacing.properties">
    <xsl:attribute name="space-before.minimum">0pt</xsl:attribute>
    <xsl:attribute name="space-before.optimum">0pt</xsl:attribute>
    <xsl:attribute name="space-before.maximum">0pt</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0pt</xsl:attribute>
    <xsl:attribute name="space-after.optimum">0pt</xsl:attribute>
    <xsl:attribute name="space-after.maximum">0pt</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="compactlist.item.spacing"
                     use-attribute-sets="no.spacing.properties">
  </xsl:attribute-set>
  
  <xsl:template match="d:varlistentry" mode="vl.as.blocks">
    <xsl:variable name="id"><xsl:call-template name="object.id"/></xsl:variable>

    <xsl:choose>
      <xsl:when test="parent::*/@spacing = 'compact'">
        <fo:block id="{$id}"
                  xsl:use-attribute-sets="compact.list.item.spacing"> 
          <xsl:call-template name="varlistentry.listitem.blocks"/>
        </fo:block>
      </xsl:when>
      <xsl:otherwise>
        <fo:block id="{$id}" xsl:use-attribute-sets="list.item.spacing"> 
          <xsl:call-template name="varlistentry.listitem.blocks"/>
        </fo:block>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:attribute-set name="variablelist.listitem.properties">
    <xsl:attribute name="space-before.minimum">0.1em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">0.25em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">0.4em</xsl:attribute>
    <xsl:attribute name="space-before.precedence">force</xsl:attribute>
    <xsl:attribute name="margin-{$direction.align.start}">0.25in</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:template name="varlistentry.listitem.blocks">
    <fo:block xsl:use-attribute-sets="variablelist.term.properties"  
              keep-together.within-column="always" 
              keep-with-next.within-column="always">
      <xsl:apply-templates select="d:term"/>
    </fo:block>
    <fo:block xsl:use-attribute-sets="variablelist.listitem.properties">
      <xsl:apply-templates select="d:listitem"/>
    </fo:block>
  </xsl:template>
  
  <!-- calloutlist ========== -->

  <xsl:attribute-set name="calloutlist.properties"
                     use-attribute-sets="normal.spacing.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="callout.properties"
                     use-attribute-sets="normal.spacing.properties">
  </xsl:attribute-set>
  
  <!-- blockquote ========== -->
  
  <xsl:attribute-set name="blockquote.properties"
                     use-attribute-sets="normal.spacing.properties">
    <xsl:attribute name="margin-{$direction.align.start}">4em</xsl:attribute>
    <xsl:attribute name="margin-{$direction.align.end}">4em</xsl:attribute>
  </xsl:attribute-set>
  
  <!-- note, tip warning, etc ========== -->
  
  <xsl:attribute-set name="admonition.spacing.properties" >
    <xsl:attribute name="space-before.minimum">1.5em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">2.17em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">2.5em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">1.5em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">2.17em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">2.5em</xsl:attribute>
    <xsl:attribute name="space-after.conditionality">retain</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="admonition.properties">
    <xsl:attribute name="font-size">0.92em</xsl:attribute>
    <xsl:attribute name="line-height">1.3</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="nongraphical.admonition.properties"
                     use-attribute-sets="admonition.spacing.properties">
    <xsl:attribute name="margin-{$direction.align.start}">3em</xsl:attribute>
    <xsl:attribute name="margin-{$direction.align.end}">3em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="graphical.admonition.properties"
                     use-attribute-sets="admonition.spacing.properties">
    <xsl:attribute name="margin-{$direction.align.start}">2em</xsl:attribute>
    <xsl:attribute name="margin-{$direction.align.end}">2em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="admonition.title.properties"
                     use-attribute-sets="normal.spacing.properties">
    <xsl:attribute name="font-size">1em</xsl:attribute>
    <xsl:attribute name="font-family"><xsl:value-of select="$title.font.family"/></xsl:attribute>
    <xsl:attribute name="font-weight">bold</xsl:attribute>
    <xsl:attribute name="font-style">normal</xsl:attribute>
    <xsl:attribute name="space-before.minimum">0pt</xsl:attribute>
    <xsl:attribute name="space-before.optimum">0pt</xsl:attribute>
    <xsl:attribute name="space-before.maximum">0pt</xsl:attribute>
  </xsl:attribute-set>
  
  <!-- sidebar ========== -->
  
  <xsl:attribute-set name="sidebar.properties"
    use-attribute-sets="admonition.properties admonition.spacing.properties">
    <xsl:attribute name="padding">1em</xsl:attribute>
    <xsl:attribute name="background-color">transparent</xsl:attribute>
    <xsl:attribute name="border-style">solid</xsl:attribute>
    <xsl:attribute name="border-width">0.5pt</xsl:attribute>
    <xsl:attribute name="border-color">black</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="sidebar.title.properties"
                     use-attribute-sets="admonition.title.properties">
    <xsl:attribute name="text-align">center</xsl:attribute>
  </xsl:attribute-set>
  
  <!-- formal/informal figure, table, etc ========== -->
  
  <xsl:attribute-set name="informal.object.properties"
                     use-attribute-sets="normal.spacing.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="formal.object.properties"
                     use-attribute-sets="normal.spacing.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="formal.title.properties"
                     use-attribute-sets="normal.spacing.properties">
    <xsl:attribute name="font-family">serif</xsl:attribute>
    <xsl:attribute name="font-weight">normal</xsl:attribute>
    <xsl:attribute name="font-style">italic</xsl:attribute>
    <xsl:attribute name="font-size">1em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0.3em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">0.5em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">0.7em</xsl:attribute>
  </xsl:attribute-set>
  
  <!-- table ========== -->
  
  <xsl:param name="table.frame.border.color">gray</xsl:param>
  <xsl:param name="table.cell.border.color">gray</xsl:param>
  
  <xsl:attribute-set name="table.cell.padding">
    <xsl:attribute name="padding-start">0.5em</xsl:attribute>
    <xsl:attribute name="padding-end">0.5em</xsl:attribute>
    <xsl:attribute name="padding-top">0.25em</xsl:attribute>
    <xsl:attribute name="padding-bottom">0.25em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="table.caption.properties"
                     use-attribute-sets="formal.title.properties">
  </xsl:attribute-set>
  
  <!-- part, chapter, appendix titles ========== -->
  
  <xsl:attribute-set name="component.title.properties">
    <xsl:attribute name="font-family"><xsl:value-of select="$title.font.family"/></xsl:attribute>
    <xsl:attribute name="font-weight">bold</xsl:attribute>
    <xsl:attribute name="font-style">normal</xsl:attribute>
    <xsl:attribute name="font-size">
      <xsl:value-of select="concat($body.font.master, 'pt * 1.8')"/>
    </xsl:attribute>
    <xsl:attribute name="line-height">1</xsl:attribute>
    <xsl:attribute name="space-before.minimum">0.35em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">0.55em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">0.75em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0.35em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">0.55em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">0.75em</xsl:attribute>
  </xsl:attribute-set>
  
  <!-- toc, lot, lof, etc ========== -->

  <xsl:attribute-set name="booklist.title.properties">
    <xsl:attribute name="font-family"><xsl:value-of select="$title.font.family"/></xsl:attribute>
    <xsl:attribute name="font-weight">bold</xsl:attribute>
    <xsl:attribute name="font-style">normal</xsl:attribute>
    <xsl:attribute name="font-size">
      <xsl:value-of select="concat($body.font.master, 'pt * 1.5')"/>
    </xsl:attribute>
    <xsl:attribute name="line-height">1</xsl:attribute>
    <xsl:attribute name="space-before.minimum">0.5em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">0.67em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">0.9em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0.5em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">0.67em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">0.9em</xsl:attribute>
  </xsl:attribute-set>

  <xsl:template name="table.of.contents.titlepage.recto">
    <fo:block xsl:use-attribute-sets="table.of.contents.titlepage.recto.style
                                      booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'TableofContents'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="list.of.tables.titlepage.recto">
    <fo:block xsl:use-attribute-sets="list.of.tables.titlepage.recto.style
                                      booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofTables'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="list.of.figures.titlepage.recto">
    <fo:block xsl:use-attribute-sets="list.of.figures.titlepage.recto.style
                                      booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofFigures'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="list.of.examples.titlepage.recto">
    <fo:block xsl:use-attribute-sets="list.of.examples.titlepage.recto.style
                                      booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofExamples'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="list.of.equations.titlepage.recto">
    <fo:block xsl:use-attribute-sets="list.of.equations.titlepage.recto.style
                                      booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofEquations'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="list.of.procedures.titlepage.recto">
    <fo:block xsl:use-attribute-sets="list.of.procedures.titlepage.recto.style
                                      booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofProcedures'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="list.of.unknowns.titlepage.recto">
    <fo:block xsl:use-attribute-sets="list.of.unknowns.titlepage.recto.style
                                      booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofUnknown'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:attribute-set name="component.booklist.title.properties">
    <xsl:attribute name="font-family"><xsl:value-of select="$title.font.family"/></xsl:attribute>
    <xsl:attribute name="font-weight">bold</xsl:attribute>
    <xsl:attribute name="font-style">normal</xsl:attribute>
    <xsl:attribute name="font-size">
      <xsl:value-of select="concat($body.font.master, 'pt * 1.2')"/>
    </xsl:attribute>
    <xsl:attribute name="line-height">1</xsl:attribute>
    <xsl:attribute name="space-before.minimum">0.5em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">0.83em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">1em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0.5em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">0.83em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">1em</xsl:attribute>
  </xsl:attribute-set>

  <xsl:template name="component.list.of.tables.titlepage.recto">
    <fo:block
      xsl:use-attribute-sets="component.list.of.tables.titlepage.recto.style
                              component.booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofTables'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="component.list.of.figures.titlepage.recto">
    <fo:block
      xsl:use-attribute-sets="component.list.of.figures.titlepage.recto.style
                              component.booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofFigures'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="component.list.of.examples.titlepage.recto">
    <fo:block
      xsl:use-attribute-sets="component.list.of.examples.titlepage.recto.style
                              component.booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofExamples'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="component.list.of.equations.titlepage.recto">
    <fo:block
      xsl:use-attribute-sets="component.list.of.equations.titlepage.recto.style
                              component.booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofEquations'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="component.list.of.procedures.titlepage.recto">
    <fo:block
      xsl:use-attribute-sets="component.list.of.procedures.titlepage.recto.style
                              component.booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofProcedures'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <xsl:template name="component.list.of.unknowns.titlepage.recto">
    <fo:block
      xsl:use-attribute-sets="component.list.of.unknowns.titlepage.recto.style
                              component.booklist.title.properties">
      <xsl:call-template name="gentext">
        <xsl:with-param name="key" select="'ListofUnknown'"/>
      </xsl:call-template>
    </fo:block>
  </xsl:template>

  <!-- section titles ========== -->
  
  <xsl:attribute-set name="section.title.properties">
    <xsl:attribute name="font-family"><xsl:value-of select="$title.font.family"/></xsl:attribute>
    <xsl:attribute name="font-weight">bold</xsl:attribute>
    <xsl:attribute name="font-style">normal</xsl:attribute>
    <xsl:attribute name="space-before.minimum">1.5em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">2em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">2.5em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0.8em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">1em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">1.2em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="section.title.level1.properties">
    <xsl:attribute name="font-size">1.3em</xsl:attribute>
    <xsl:attribute name="line-height">1</xsl:attribute>
    <xsl:attribute name="space-before.minimum">1em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">1.54em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">2em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0.5em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">0.77em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">1em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="section.title.level2.properties">
    <xsl:attribute name="font-size">1.2em</xsl:attribute>
    <xsl:attribute name="line-height">1</xsl:attribute>
    <xsl:attribute name="space-before.minimum">1em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">1.67em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">2em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0.5em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">0.83em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">1em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="section.title.level3.properties">
    <xsl:attribute name="font-size">1.1em</xsl:attribute>
    <xsl:attribute name="line-height">1.1</xsl:attribute>
    <xsl:attribute name="space-before.minimum">1em</xsl:attribute>
    <xsl:attribute name="space-before.optimum">1.82em</xsl:attribute>
    <xsl:attribute name="space-before.maximum">2em</xsl:attribute>
    <xsl:attribute name="space-after.minimum">0.6em</xsl:attribute>
    <xsl:attribute name="space-after.optimum">0.91em</xsl:attribute>
    <xsl:attribute name="space-after.maximum">1.2em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="section.title.level4.properties">
    <xsl:attribute name="font-size">1em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="section.title.level5.properties">
    <xsl:attribute name="font-size">1em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="section.title.level6.properties">
    <xsl:attribute name="font-size">1em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="refentry.title.properties"
                     use-attribute-sets="section.title.properties">
    <xsl:attribute name="font-size">1em</xsl:attribute>
  </xsl:attribute-set>
  
  <!-- qanda set titles ========== -->
  
  <xsl:attribute-set name="qanda.title.properties"
                     use-attribute-sets="section.title.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="qanda.title.level1.properties"
                     use-attribute-sets="section.title.level1.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="qanda.title.level2.properties"
                     use-attribute-sets="section.title.level2.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="qanda.title.level3.properties"
                     use-attribute-sets="section.title.level3.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="qanda.title.level4.properties"
                     use-attribute-sets="section.title.level4.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="qanda.title.level5.properties"
                     use-attribute-sets="section.title.level5.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="qanda.title.level6.properties"
                     use-attribute-sets="section.title.level6.properties">
  </xsl:attribute-set>

  <!-- header, footer content ========== -->
  
  <xsl:attribute-set name="header.footer.properties">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
    <xsl:attribute name="font-size">
      <xsl:value-of select="concat($body.font.master, 'pt * 0.8')"/>
    </xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="header.content.properties"
                     use-attribute-sets="header.footer.properties">
  </xsl:attribute-set>
  
  <xsl:attribute-set name="footer.content.properties"
                     use-attribute-sets="header.footer.properties">
  </xsl:attribute-set>
  
  <!-- footnote ========== -->

  <xsl:attribute-set name="footnote.sep.leader.properties">
    <xsl:attribute name="rule-thickness">0.5pt</xsl:attribute>
    <xsl:attribute name="leader-length">4em</xsl:attribute>
  </xsl:attribute-set>

  <!-- revhistory ========== -->
  
  <xsl:attribute-set name="revhistory.table.properties"
                     use-attribute-sets="normal.spacing.properties">
    <xsl:attribute name="border-top-style">solid</xsl:attribute>
    <xsl:attribute name="border-top-width">0.5pt</xsl:attribute>
    <xsl:attribute name="border-top-color">gray</xsl:attribute>
    <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
    <xsl:attribute name="border-bottom-width">0.5pt</xsl:attribute>
    <xsl:attribute name="border-bottom-color">gray</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="revhistory.table.cell.properties">
    <xsl:attribute name="text-align">left</xsl:attribute>
    <xsl:attribute name="font-family">serif</xsl:attribute>
    <xsl:attribute name="font-weight">normal</xsl:attribute>
    <xsl:attribute name="font-size">0.92em</xsl:attribute>
    <xsl:attribute name="line-height">1.3</xsl:attribute>
    <xsl:attribute name="padding-top">0.125em</xsl:attribute>
    <xsl:attribute name="padding-bottom">0.125em</xsl:attribute>
  </xsl:attribute-set>
  
  <xsl:attribute-set name="revhistory.title.properties"
                     use-attribute-sets="revhistory.table.cell.properties">
    <xsl:attribute name="font-style">italic</xsl:attribute>
    <xsl:attribute name="font-size">1em</xsl:attribute>
  </xsl:attribute-set>
  
  <!-- Font style changes ========== -->

  <xsl:template match="d:guibutton">
    <xsl:call-template name="inline.boldseq"/>
  </xsl:template>

  <xsl:template match="d:guiicon">
    <xsl:call-template name="inline.boldseq"/>
  </xsl:template>

  <xsl:template match="d:guilabel">
    <xsl:call-template name="inline.boldseq"/>
  </xsl:template>

  <xsl:template match="d:guimenu">
    <xsl:call-template name="inline.boldseq"/>
  </xsl:template>

  <xsl:template match="d:guimenuitem">
    <xsl:call-template name="inline.boldseq"/>
  </xsl:template>

  <xsl:template match="d:guisubmenu">
    <xsl:call-template name="inline.boldseq"/>
  </xsl:template>

  <xsl:template match="d:abbrev">
    <xsl:call-template name="inline.boldseq"/>
  </xsl:template>

  <xsl:template match="d:acronym">
    <xsl:call-template name="inline.boldseq"/>
  </xsl:template>

</xsl:stylesheet>

