<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:d="http://docbook.org/ns/docbook"
                version='1.0'>

<!-- '0' (means: none) or '1' (means: all) or 
     a space separated list containing one or more of 
     'tab-position', 'label-format', 'render-note-as-table', 
     'render-sidebar-as-table'. -->
<xsl:param name="xfc.extensions" select="0"/>

<!-- provisional-distance-between-starts is:
     $orderedlist.label.width + (nesting-1)*$nested.orderedlist.label.width -->
<xsl:param name="nested.orderedlist.label.width" select="'1em'"/>

<!-- ======================================================================
     Called by <fo:leader xsl:use-attribute-sets="toc.leader.properties">. 
     Add extension attributes xfc:tab-position and xfc:tab-align. 
     ====================================================================== -->

<xsl:template name="xfcTabAttributes">
  <xsl:if test="$xfc.extensions = 1 or 
                contains($xfc.extensions, 'tab-position')">
    <xsl:attribute name="xfc:tab-position"
      xmlns:xfc="http://www.xmlmind.com/foconverter/xsl/extensions">
      <xsl:value-of select="concat('-(', $toc.indent.width, 'pt + 7.5mm)')"/>
    </xsl:attribute>

    <xsl:attribute xmlns:xfc="http://www.xmlmind.com/foconverter/xsl/extensions"
                   name="xfc:tab-align">right</xsl:attribute>
  </xsl:if>
</xsl:template>

<!-- ======================================================================
     Called by the "sidebar" template in block.xsl. 
     Called by "graphical.admonition" and "nongraphical.admonition" 
     in admon.xsl.
     Add extension attribute xfc:render-as-table. 
     ====================================================================== -->

<xsl:template name="xfcRenderAsTable">
  <xsl:param name="which" select="''"/>

  <xsl:if test="$xfc.extensions = 1 or 
                contains($xfc.extensions, $which)">
    <xsl:attribute xmlns:xfc="http://www.xmlmind.com/foconverter/xsl/extensions"
                   name="xfc:render-as-table">true</xsl:attribute>
  </xsl:if>
</xsl:template>

<xsl:template name="xfcRenderNoteAsTable">
  <xsl:call-template name="xfcRenderAsTable">
    <xsl:with-param name="which" select="'render-note-as-table'"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="xfcRenderSidebarAsTable">
  <xsl:call-template name="xfcRenderAsTable">
    <xsl:with-param name="which" select="'render-sidebar-as-table'"/>
  </xsl:call-template>
</xsl:template>

<!-- ======================================================================
     Context node is orderedlist. 
     Add extension attribute xfc:label-format.
     ====================================================================== -->

<xsl:template name="orderedlistNesting">
  <xsl:param name="ol" select="''"/>
  <xsl:param name="nesting" select="1"/>

  <!-- inheritnum works across sections, notes, tables, etc. Strange. -->
  <xsl:variable name="listitemParent" 
                select="$ol/ancestor::d:listitem[parent::d:orderedlist][1]"/>
  <xsl:choose>
    <xsl:when test="$listitemParent">
      <xsl:choose>
        <xsl:when test="$listitemParent/parent::*/@inheritnum = 'inherit'">
          <xsl:call-template name="orderedlistNesting">
            <xsl:with-param name="ol" select="$listitemParent/parent::*"/>
            <xsl:with-param name="nesting" select="$nesting + 1"/>
          </xsl:call-template>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="$nesting + 1"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$nesting"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="xfcLabelFormat">
  <xsl:param name="userLabelWidth" select="$orderedlist.label.width"/>

  <xsl:if test="$xfc.extensions = 1 or 
                contains($xfc.extensions, 'label-format')">
      <xsl:variable name="labelFormat">
        <xsl:variable name="num">
          <xsl:choose>
            <xsl:when test="@numeration = 'arabic'">decimal</xsl:when>
            <xsl:when test="@numeration = 'loweralpha'">lower-alpha</xsl:when>
            <xsl:when test="@numeration = 'lowerroman'">lower-roman</xsl:when>
            <xsl:when test="@numeration = 'upperalpha'">upper-alpha</xsl:when>
            <xsl:when test="@numeration = 'upperroman'">upper-roman</xsl:when>
            <xsl:otherwise>decimal</xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:variable name="start">
          <xsl:if test="@continuation = 'continues'">
            <xsl:variable name="startNum">
              <xsl:call-template name="orderedlist-starting-number"/>
            </xsl:variable>

            <xsl:if test="$startNum &gt; 1">
              <xsl:value-of select="concat(';start=', $startNum)"/>
            </xsl:if>
          </xsl:if>
        </xsl:variable>

        <xsl:variable name="inherit">
          <xsl:if test="@inheritnum = 'inherit'">
            <xsl:value-of select="';inherit'"/>
          </xsl:if>
        </xsl:variable>

        <xsl:value-of select="concat('%{', $num, $start, $inherit, '}.')"/>
      </xsl:variable>

      <!--
      <xsl:message><xsl:value-of select="concat('xfc:label-format=', 
                                                $labelFormat)"/></xsl:message>
      -->

      <xsl:attribute name="xfc:label-format"
        xmlns:xfc="http://www.xmlmind.com/foconverter/xsl/extensions">
        <xsl:value-of select="$labelFormat"/>
      </xsl:attribute>
  </xsl:if>

  <!-- Fix provisional-distance-between-starts for ALL XSL-FO processors. -->

  <xsl:variable name="nesting">
    <xsl:call-template name="orderedlistNesting">
      <xsl:with-param name="ol" select="."/>
      <xsl:with-param name="olNesting" select="1"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="labelWidth">
    <xsl:choose>
      <xsl:when test="@inheritnum = 'inherit' and $nesting &gt; 1">
        <xsl:value-of 
          select="concat($userLabelWidth, ' + (', 
                         ($nesting - 1), ' * ', 
                         $nested.orderedlist.label.width, ')')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$userLabelWidth"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!--
  <xsl:message><xsl:value-of 
    select="concat('nesting=', $nesting,
                   ', provisional-distance-between-starts=', 
                   $labelWidth)"/></xsl:message>
  -->

  <xsl:attribute name="provisional-distance-between-starts">
    <xsl:value-of select="$labelWidth"/>
  </xsl:attribute>
</xsl:template>

</xsl:stylesheet>
