<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2017 XMLmind Software. All rights reserved.
|
| Author: Hussein Shafie (hussein@xmlmind.com)
|
| This file is part of the XMLmind XML Editor project.
| For conditions of distribution and use, see the accompanying legal.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:html="http://www.w3.org/1999/xhtml"
                exclude-result-prefixes="xs html"
                version="2.0">

  <xsl:import href="params.xsl"/>
  <xsl:import href="helpers.xsl"/>
  <xsl:import href="cssStyles.xsl"/>
  <xsl:import href="pagination.xsl"/>
  <xsl:import href="body.xsl"/>
  <xsl:import href="table.xsl"/>

  <!-- Output ============================================================ -->

  <xsl:output method="xml" encoding="UTF-8" indent="no"/>

  <xsl:preserve-space elements="html:pre html:style html:script"/>

  <!-- The root element ================================================== -->

  <xsl:attribute-set name="html">
    <xsl:attribute name="font-family" select="$font-family"/>
    <xsl:attribute name="font-size" select="$base-font-size"/>
    <xsl:attribute name="line-height">1.2</xsl:attribute>
    <xsl:attribute name="text-align"
                  select="if ($justified eq 'yes') then 'justify' else 'left'"/>
  </xsl:attribute-set>

  <xsl:template match="html:html">
    <fo:root>
      <xsl:call-template name="localizationAttributes"/>

      <xsl:call-template name="layoutMasterSet"/>

      <fo:page-sequence>
        <xsl:call-template name="configurePageSequence">
          <xsl:with-param name="documentTitle" 
                          select="normalize-space(html:head/html:title)"/>
        </xsl:call-template>

        <fo:flow flow-name="xsl-region-body">
          <fo:block xsl:use-attribute-sets="html">
            <xsl:call-template name="commonAttributes"/>
            <xsl:apply-templates/>
          </fo:block>

          <fo:block id="__EOF" hyphenate="false"/>
        </fo:flow>
      </fo:page-sequence>
    </fo:root>
  </xsl:template>

  <!-- Document metadata ================================================= -->
  <!-- title, base, meta, script, style, noscript not rendered. -->

  <xsl:template match="html:head" />

  <xsl:template match="html:style" />

  <!-- Scripting ========================================================= -->
  <!-- not rendered. -->

  <xsl:template match="html:script" />

  <xsl:template match="html:template" />

  <!-- The noscript element must not be used in XHTML5 documents. -->
  <!--LEGACY-->
  <xsl:template match="html:noscript">
    <fo:block>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

  <!-- body ============================================================== -->

  <xsl:attribute-set name="body">
  </xsl:attribute-set>

  <xsl:template match="html:body">
    <fo:block xsl:use-attribute-sets="body">
      <xsl:choose>
        <xsl:when test="$root-id ne ''">
          <xsl:variable name="root" select="//*[@id eq $root-id]"/>
          <xsl:choose>
            <xsl:when test="exists($root)">
              <!-- Apply body CSS styles and not root div CSS styles. -->
              <xsl:call-template name="cssStyles"/>

              <xsl:for-each select="$root">
                <xsl:call-template name="commonAttributes"/>
                <xsl:apply-templates select="./node()"/>
              </xsl:for-each>
            </xsl:when>
            
            <xsl:otherwise>
              <xsl:message terminate="yes">
                <xsl:text>Do not find any element having '</xsl:text>
                <xsl:value-of select="$root-id"/>
                <xsl:text>' as its ID.</xsl:text>
              </xsl:message>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:otherwise>
          <!--LEGACY-->
          <xsl:call-template name="bgcolorAttribute"/>
          <xsl:call-template name="textAttribute"/>
          <xsl:call-template name="cssStyles"/>

          <xsl:call-template name="commonAttributes"/>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
    </fo:block>
  </xsl:template>

  <!--LEGACY-->
  <xsl:template name="textAttribute">
    <xsl:if test="exists(@text)">
      <xsl:attribute name="color" select="string(@text)"/>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
