<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2017 XMLmind Software. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind Ebook Compiler project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="http://www.w3.org/1999/xhtml"
                xmlns:u="http://www.xmlmind.com/namespace/ebookc"
                xpath-default-namespace="http://www.w3.org/1999/xhtml"
                exclude-result-prefixes="u"
                version="2.0">

  <xsl:output name="tocOutputFormat"
              method="xhtml" encoding="UTF-8" indent="no"
              omit-xml-declaration="yes" include-content-type="no"/>

  <xsl:template name="generateTOC">
    <xsl:result-document href="{resolve-uri('__toc__.xhtml', base-uri())}"
                         format="tocOutputFormat">

      <xsl:text
        disable-output-escaping="yes">&lt;!DOCTYPE html&gt;&#xA;</xsl:text>

      <html>
        <xsl:copy-of select="@lang"/>

        <head>
          <meta charset="UTF-8"/>
          <title><xsl:value-of select="normalize-space(./head/title)"/></title>
        </head>

        <body>
          <nav epub:type="toc" xmlns:epub="http://www.idpf.org/2007/ops">
            <h1><xsl:value-of select="u:localize('toc', .)"/></h1>

            <xsl:variable name="entries" 
              select="./body/ul[u:has(@class, 'role-booklist-entries')]"/>
            <xsl:choose>
              <xsl:when
                test="$omit-toc-root eq 'yes' and 
                      count($entries/*) eq 1 and
                      count($entries/*/ul[u:has(@class, 'role-booklist-entries')]) eq 1">
                <xsl:apply-templates 
                  select="$entries/*/ul[u:has(@class, 'role-booklist-entries')]"
                  mode="TOC"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:apply-templates select="$entries" mode="TOC"/>
              </xsl:otherwise>
            </xsl:choose>
          </nav>
        </body>
      </html>

    </xsl:result-document>
  </xsl:template>

  <xsl:template match="ul[u:has(@class, 'role-booklist-entries')]" mode="TOC">
    <ol>
      <xsl:apply-templates mode="TOC"/>
    </ol>
  </xsl:template>

  <xsl:template match="li[u:has(@class, 'role-booklist-entry')]" mode="TOC">
    <li>
      <xsl:apply-templates mode="TOC"/>
    </li>
  </xsl:template>

  <xsl:template match="li[u:has(@class,'role-booklist-entry')]/
                       a[u:matches(@class, 'role-', '-ref')]"
                mode="TOC">
    <a>
      <xsl:copy-of select="@href"/>
      <xsl:copy-of select="./node()"/>
    </a>
  </xsl:template>

</xsl:stylesheet>
