<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2017 XMLmind Software. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind Ebook Compiler project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="http://www.daisy.org/z3986/2005/ncx/"
                xmlns:u="http://www.xmlmind.com/namespace/ebookc"
                xpath-default-namespace="http://www.w3.org/1999/xhtml"
                exclude-result-prefixes="u"
                version="2.0">

  <!-- It's not mandatory to have a DOCTYPE
       doctype-public="-//NISO//DTD ncx 2005-1//EN"
       doctype-system="http://www.daisy.org/z3986/2005/ncx-2005-1.dtd" -->

  <xsl:output name="tocNCXOutputFormat"
    method="xml" encoding="UTF-8" indent="yes"/>

  <xsl:template name="generateTOC_NCX">
    <xsl:result-document href="{resolve-uri('toc.ncx', base-uri())}"
                         format="tocNCXOutputFormat">

      <xsl:variable name="tocList0" 
                    select="./body/ul[u:has(@class, 'role-booklist-entries')]"/>
      <xsl:variable name="tocList"
        select="if ($omit-toc-root eq 'yes' and 
                    count($tocList0/*) eq 1 and
                    count($tocList0/*/ul[u:has(@class, 'role-booklist-entries')]) eq 1)
                then $tocList0/*/ul[u:has(@class, 'role-booklist-entries')]
                else $tocList0" />

      <xsl:variable name="tocEntries"
                    select="$tocList//li[u:has(@class, 'role-booklist-entry')]"/>

      <xsl:variable name="tocDepth" 
        select="max(for $e in $tocEntries
                    return
                        count($e/ancestor-or-self::li[u:has(@class, 'role-booklist-entry')]))"/>

      <ncx version="2005-1">
        <head>
          <meta name="dtb:uid" content="{$epubIdentifier}"/>
          <meta name="dtb:depth" content="{$tocDepth}"/>
          <meta name="dtb:totalPageCount" content="0"/>
          <meta name="dtb:maxPageNumber" content="0"/>
        </head>

        <docTitle>
          <text>
            <xsl:value-of select="normalize-space(./head/title)"/>
          </text>
        </docTitle>

        <navMap>
          <xsl:apply-templates select="$tocList" mode="TOC_NCX">
            <xsl:with-param name="tocEntries" select="$tocEntries"
                            tunnel="yes"/>
          </xsl:apply-templates>
        </navMap>
      </ncx>
    </xsl:result-document>
  </xsl:template>

  <xsl:template match="ul[u:has(@class, 'role-booklist-entries')]"
                mode="TOC_NCX">
    <xsl:apply-templates mode="TOC_NCX"/>
  </xsl:template>

  <xsl:template match="li[u:has(@class, 'role-booklist-entry')]"
                mode="TOC_NCX">
    <xsl:param name="tocEntries" select="()" tunnel="yes"/>

    <xsl:variable name="playOrder" select="u:indexOfNode($tocEntries, .)"/>

    <navPoint id="{string(@id)}" playOrder="{$playOrder}">
      <xsl:apply-templates mode="TOC_NCX"/>
    </navPoint>
  </xsl:template>

  <xsl:template match="li[u:has(@class,'role-booklist-entry')]/
                       a[u:matches(@class, 'role-', '-ref')]"
                mode="TOC_NCX">
    <navLabel>
      <text>
        <xsl:value-of select="normalize-space(.)"/>
      </text>
    </navLabel>
    <content src="{string(@href)}"/>
  </xsl:template>

</xsl:stylesheet>
