<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2017-2019 XMLmind Software. All rights reserved.
|
| Author: Hussein Shafie (hussein@xmlmind.com)
|
| This file is part of the XMLmind Ebook Compiler project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:html="http://www.w3.org/1999/xhtml"
                xmlns:h="http://www.xmlmind.com/namespace/xsl/fo/helpers"
                xmlns:u="http://www.xmlmind.com/namespace/ebookc"
                xmlns:URI="java:com.xmlmind.ebookc.xslt.URI"
                exclude-result-prefixes="xs html h u URI"
                version="2.0">

  <xsl:template name="layoutMasterSet">
    <fo:layout-master-set>
      <!-- simple-page-masters =========================================== -->

      <!-- blank-page ========== -->

      <fo:simple-page-master master-name="blank-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'blank'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"/>
        <fo:region-after extent="{$footer-height}"/>
      </fo:simple-page-master>

      <!-- title-page ========== -->

      <fo:simple-page-master master-name="title-first-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <xsl:choose>
          <xsl:when test="$foProcessor eq 'XEP'">
            <!-- The border around the region-body is a XEP extension 
                 (Border and Padding on Regions).
                 In both XSL 1.0 and 1.1, the values of the padding and 
                 border-width traits must be "0".  -->
            <fo:region-body margin-top="{$body-top-margin}"
                            margin-bottom="{$body-bottom-margin}">
              <xsl:call-template name="addWatermark">
                <xsl:with-param name="pageMasterType" select="'title'"/>
              </xsl:call-template>
            </fo:region-body>
          </xsl:when>
          <xsl:otherwise>
            <fo:region-body margin-top="{$body-top-margin}"
                            margin-bottom="{$body-bottom-margin}">
              <xsl:call-template name="addWatermark">
                <xsl:with-param name="pageMasterType" select="'title'"/>
              </xsl:call-template>
            </fo:region-body>
          </xsl:otherwise>
        </xsl:choose>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="title-odd-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'title'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="title-even-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'title'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- toc-page ========== -->

      <fo:simple-page-master master-name="toc-first-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'toc'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="toc-odd-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'toc'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="toc-even-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'toc'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- booklist-page ========== -->

      <fo:simple-page-master master-name="booklist-first-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'booklist'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="booklist-odd-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'booklist'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="booklist-even-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'booklist'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- frontmatter-page ========== -->

      <fo:simple-page-master master-name="frontmatter-first-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'frontmatter'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                          display-align="after"
                          region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="frontmatter-odd-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'frontmatter'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="frontmatter-even-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'frontmatter'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- body-page ========== -->

      <fo:simple-page-master master-name="body-first-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'body'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="body-odd-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'body'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                          display-align="after"
                          region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="body-even-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'body'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- backmatter-page ========== -->

      <fo:simple-page-master master-name="backmatter-first-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'backmatter'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="backmatter-odd-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'backmatter'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="backmatter-even-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'backmatter'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- index-page ========== -->

      <fo:simple-page-master master-name="index-first-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"
                        column-count="{$index-column-count}" 
                        column-gap="{$index-column-gap}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'index'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="index-odd-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"
                        column-count="{$index-column-count}" 
                        column-gap="{$index-column-gap}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'index'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="index-even-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"
                        column-count="{$index-column-count}" 
                        column-gap="{$index-column-gap}">
          <xsl:call-template name="addWatermark">
            <xsl:with-param name="pageMasterType" select="'index'"/>
          </xsl:call-template>
        </fo:region-body>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- page-sequence-masters (two-sided) ============================= -->

      <!-- title ========== -->

      <fo:page-sequence-master master-name="title">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="title-first-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="title-odd-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="title-even-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- toc ========== -->

      <fo:page-sequence-master master-name="toc">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="toc-first-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="toc-odd-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="toc-even-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- booklist ========== -->

      <fo:page-sequence-master master-name="booklist">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="booklist-first-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="booklist-odd-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="booklist-even-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- frontmatter ========== -->

      <fo:page-sequence-master master-name="frontmatter">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="frontmatter-first-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="frontmatter-odd-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="frontmatter-even-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- body ========== -->

      <fo:page-sequence-master master-name="body">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="body-first-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="body-odd-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="body-even-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- backmatter ========== -->

      <fo:page-sequence-master master-name="backmatter">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="backmatter-first-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="backmatter-odd-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="backmatter-even-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- index ========== -->

      <fo:page-sequence-master master-name="index">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="index-first-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="index-odd-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="index-even-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

    </fo:layout-master-set>
  </xsl:template>

  <xsl:template name="addWatermark">
    <xsl:param name="pageMasterType" select="''"/>

    <xsl:if test="$watermark-image ne '' and
                  (index-of($watermarkList, 'all') ge 1 or 
                   index-of($watermarkList, $pageMasterType) ge 1)">
      <xsl:attribute name="background-image"
        select="concat('url(', resolve-uri($watermark-image, URI:userDirectory()), ')')"/>
      <xsl:attribute name="background-attachment" select="'fixed'"/>
      <xsl:attribute name="background-repeat" select="'no-repeat'"/>
      <xsl:attribute name="background-position-horizontal" select="'center'"/>
      <xsl:attribute name="background-position-vertical" select="'center'"/>
    </xsl:if>
  </xsl:template>

  <!-- configurePageSequence ============================================= -->

  <xsl:template name="configurePageSequence">
    <xsl:param name="documentDivisions" select="()" tunnel="yes"/>

    <xsl:variable name="pos" select="h:indexOfNode($documentDivisions, .)"/>

    <xsl:variable name="sequenceName">
      <xsl:if test="number($pos) ge 1">
        <xsl:call-template name="pageSequenceName"/>
      </xsl:if>
    </xsl:variable>

    <xsl:variable name="prevSequenceName">
      <xsl:if test="number($pos) gt 1">
        <xsl:for-each select="$documentDivisions[$pos - 1]">
          <xsl:call-template name="pageSequenceName"/>
        </xsl:for-each>
      </xsl:if>
    </xsl:variable>

    <xsl:variable name="nextSequenceName">
      <xsl:if test="number($pos) lt count($documentDivisions)">
        <xsl:for-each select="$documentDivisions[$pos + 1]">
          <xsl:call-template name="pageSequenceName"/>
        </xsl:for-each>
      </xsl:if>
    </xsl:variable>

    <xsl:call-template name="configureMasterReference">
      <xsl:with-param name="sequenceName" select="$sequenceName"/>
      <xsl:with-param name="prevSequenceName" select="$prevSequenceName"/>
      <xsl:with-param name="nextSequenceName" select="$nextSequenceName"/>
    </xsl:call-template>

    <xsl:call-template name="configureHyphenate">
      <xsl:with-param name="sequenceName" select="$sequenceName"/>
      <xsl:with-param name="prevSequenceName" select="$prevSequenceName"/>
      <xsl:with-param name="nextSequenceName" select="$nextSequenceName"/>
    </xsl:call-template>

    <xsl:call-template name="configureInitialPageNumber">
      <xsl:with-param name="sequenceName" select="$sequenceName"/>
      <xsl:with-param name="prevSequenceName" select="$prevSequenceName"/>
      <xsl:with-param name="nextSequenceName" select="$nextSequenceName"/>
    </xsl:call-template>

    <xsl:call-template name="configureForcePageCount">
      <xsl:with-param name="sequenceName" select="$sequenceName"/>
      <xsl:with-param name="prevSequenceName" select="$prevSequenceName"/>
      <xsl:with-param name="nextSequenceName" select="$nextSequenceName"/>
    </xsl:call-template>

    <xsl:call-template name="configureFormat">
      <xsl:with-param name="sequenceName" select="$sequenceName"/>
      <xsl:with-param name="prevSequenceName" select="$prevSequenceName"/>
      <xsl:with-param name="nextSequenceName" select="$nextSequenceName"/>
    </xsl:call-template>

    <xsl:call-template name="configureFootnoteSeparator">
      <xsl:with-param name="sequenceName" select="$sequenceName"/>
      <xsl:with-param name="prevSequenceName" select="$prevSequenceName"/>
      <xsl:with-param name="nextSequenceName" select="$nextSequenceName"/>
    </xsl:call-template>

    <xsl:call-template name="configureHeadersAndFooters">
      <xsl:with-param name="sequenceName" select="$sequenceName"/>
      <xsl:with-param name="prevSequenceName" select="$prevSequenceName"/>
      <xsl:with-param name="nextSequenceName" select="$nextSequenceName"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="pageSequenceName">
    <xsl:choose>
      <xsl:when test="u:has(@class, 'role-book')">title</xsl:when>
      <xsl:when test="u:has(@class, 'role-toc')">toc</xsl:when>
      <xsl:when test="u:has(@class, 'role-index')">index</xsl:when>
      <xsl:when test="u:has(@class,
                            ('role-lot', 'role-lof', 
                             'role-lox', 'role-loe'))">booklist</xsl:when>
      <xsl:when test="u:has(@class,
                            ('role-part', 'role-appendices', 
                             'role-chapter', 'role-appendix'))">body</xsl:when>
      <xsl:when test="u:has(@class, 'role-section1')">
        <xsl:choose>
          <xsl:when
           test="parent::*[u:has(@class, 'role-frontmatter')]">frontmatter</xsl:when>
          <xsl:when
           test="parent::*[u:has(@class, 'role-backmatter')]">backmatter</xsl:when>
        </xsl:choose>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <!-- configureMasterReference ========================================== -->

  <xsl:template name="configureMasterReference">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="prevSequenceName" select="''"/>
    <xsl:param name="nextSequenceName" select="''"/>

    <xsl:attribute name="master-reference"
                   select="if ($two-sided eq 'yes') 
                           then $sequenceName
                           else concat($sequenceName, '-odd-page')"/>
  </xsl:template>

  <!-- configureHyphenate ================================================ -->

  <xsl:template name="configureHyphenate">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="prevSequenceName" select="''"/>
    <xsl:param name="nextSequenceName" select="''"/>

    <xsl:choose>
      <xsl:when test="$sequenceName = ('body', 'frontmatter', 'backmatter')">
        <xsl:attribute name="hyphenate" 
                       select="if ($hyphenate eq 'yes') 
                               then 'true'
                               else 'false'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="hyphenate" select="'false'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- configureInitialPageNumber ======================================== -->

  <xsl:template name="configureInitialPageNumber">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="prevSequenceName" select="''"/>
    <xsl:param name="nextSequenceName" select="''"/>

    <xsl:variable name="frontBackMatter" 
      select="('frontmatter', 'backmatter', 'toc', 'index', 'booklist')"/>

    <xsl:attribute name="initial-page-number">
      <xsl:choose>
        <xsl:when test="$sequenceName eq 'title'">1</xsl:when>

        <xsl:when test="$sequenceName = $frontBackMatter and
                        not($prevSequenceName = $frontBackMatter)">1</xsl:when>

        <xsl:when test="$sequenceName eq 'body' and
                        $prevSequenceName ne 'body'">1</xsl:when>

        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="$two-sided eq 'yes'">auto-odd</xsl:when>
            <xsl:otherwise>auto</xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>

  <!-- configureForcePageCount =========================================== -->

  <xsl:template name="configureForcePageCount">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="prevSequenceName" select="''"/>
    <xsl:param name="nextSequenceName" select="''"/>

    <xsl:attribute name="force-page-count" 
                   select="if ($two-sided eq 'yes' and $nextSequenceName ne '') 
                           then 'end-on-even'
                           else 'no-force'"/>
  </xsl:template>

  <!-- configureFormat =================================================== -->

  <xsl:template name="configureFormat">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="prevSequenceName" select="''"/>
    <xsl:param name="nextSequenceName" select="''"/>

    <xsl:attribute name="format" 
                   select="if ($sequenceName eq 'body') then '1' else 'i'"/>
  </xsl:template>

  <!-- configureFootnoteSeparator ======================================== -->

  <xsl:template name="configureFootnoteSeparator">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="prevSequenceName" select="''"/>
    <xsl:param name="nextSequenceName" select="''"/>

    <xsl:if test="$sequenceName = ('frontmatter', 'body', 'backmatter')">
      <xsl:call-template name="footnoteSeparator"/>
    </xsl:if>
  </xsl:template>

  <xsl:attribute-set name="footnote-separator">
    <xsl:attribute name="leader-pattern">rule</xsl:attribute>
    <xsl:attribute name="leader-length">4em</xsl:attribute>
    <xsl:attribute name="rule-style">solid</xsl:attribute>
    <xsl:attribute name="color">#808080</xsl:attribute>
    <xsl:attribute name="rule-thickness">0.25pt</xsl:attribute>
  </xsl:attribute-set>

  <xsl:template name="footnoteSeparator">
    <fo:static-content flow-name="xsl-footnote-separator">
      <fo:block>
        <fo:leader xsl:use-attribute-sets="footnote-separator"/>
      </fo:block>
    </fo:static-content>
  </xsl:template>

  <!-- configureHeadersAndFooters ======================================== -->

  <xsl:template name="configureHeadersAndFooters">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="prevSequenceName" select="''"/>
    <xsl:param name="nextSequenceName" select="''"/>

    <xsl:if test="$sequenceName ne 'title'">
      <xsl:call-template name="headersAndFooters">
        <xsl:with-param name="sequenceName" select="$sequenceName"/>
        <xsl:with-param name="prevSequenceName" select="$sequenceName"/>
        <xsl:with-param name="nextSequenceName" select="$sequenceName"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="headersAndFooters">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="prevSequenceName" select="''"/>
    <xsl:param name="nextSequenceName" select="''"/>
    <xsl:param name="documentTitle" select="''" tunnel="yes"/>
    <xsl:param name="documentDate" select="''" tunnel="yes"/>

    <xsl:variable name="chapterTitle">
      <fo:retrieve-marker retrieve-class-name="chapter-title"/>
    </xsl:variable>

    <xsl:variable name="section1Title">
      <fo:retrieve-marker retrieve-class-name="section1-title"/>
    </xsl:variable>

    <xsl:variable name="divTitle">
      <fo:retrieve-marker retrieve-class-name="division-title"/>
    </xsl:variable>

    <xsl:variable name="pageNumber">
      <fo:page-number/>
    </xsl:variable>

    <xsl:variable name="pageCount">
      <xsl:choose>
        <xsl:when test="exists(parent::html:div[u:has(@class, 'role-frontmatter')])">
          <fo:page-number-citation ref-id="__EOFM"/>
        </xsl:when>
        <xsl:when test="$sequenceName eq 'body'">
          <fo:page-number-citation ref-id="__EOBO"/>
        </xsl:when>
        <xsl:when test="exists(parent::html:div[u:has(@class, 'role-backmatter')])">
          <fo:page-number-citation ref-id="__EOBM"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="lineBreak">
      <fo:block/>
    </xsl:variable>

    <!-- For a single-sided doc, first-XXX and even-XXX flows don't exist -->
    <xsl:if test="$two-sided eq 'yes'">
      <xsl:variable name="hl1" 
        select="h:evalHeaderOrFooter($header-left,
                                     $sequenceName, 'first', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>
      <xsl:variable name="hc1" 
        select="h:evalHeaderOrFooter($header-center,
                                     $sequenceName, 'first', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>
      <xsl:variable name="hr1" 
        select="h:evalHeaderOrFooter($header-right,
                                     $sequenceName, 'first', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>

      <xsl:variable name="hl1W" 
        select="h:evalHeaderOrFooterWidth($header-left-width,
                                          $sequenceName, 'first')"/>
      <xsl:variable name="hc1W" 
        select="h:evalHeaderOrFooterWidth($header-center-width,
                                          $sequenceName, 'first')"/>
      <xsl:variable name="hr1W" 
        select="h:evalHeaderOrFooterWidth($header-right-width,
                                          $sequenceName, 'first')"/>

      <xsl:if test="exists($hl1) or exists($hc1) or exists($hr1)">
        <fo:static-content flow-name="first-page-header">
          <xsl:call-template name="tabularHeader">
            <xsl:with-param name="left" select="$hl1"/>
            <xsl:with-param name="center" select="$hc1"/>
            <xsl:with-param name="right" select="$hr1"/>
            <xsl:with-param name="isFooter" select="false()"/>
            <xsl:with-param name="leftWidth" select="$hl1W"/>
            <xsl:with-param name="centerWidth" select="$hc1W"/>
            <xsl:with-param name="rightWidth" select="$hr1W"/>
          </xsl:call-template>
        </fo:static-content>
      </xsl:if>
    </xsl:if>

    <xsl:variable name="hlo" 
      select="h:evalHeaderOrFooter($header-left,
                                   $sequenceName, 'odd', 
                                   $documentTitle, $documentDate,
                                   $chapterTitle, $section1Title, $divTitle,
                                   $pageNumber, $pageCount,
                                   $lineBreak)"/>
    <xsl:variable name="hco" 
      select="h:evalHeaderOrFooter($header-center,
                                   $sequenceName, 'odd', 
                                   $documentTitle, $documentDate,
                                   $chapterTitle, $section1Title, $divTitle,
                                   $pageNumber, $pageCount,
                                   $lineBreak)"/>
    <xsl:variable name="hro" 
      select="h:evalHeaderOrFooter($header-right,
                                   $sequenceName, 'odd', 
                                   $documentTitle, $documentDate,
                                   $chapterTitle, $section1Title, $divTitle,
                                   $pageNumber, $pageCount,
                                   $lineBreak)"/>

    <xsl:variable name="hloW" 
      select="h:evalHeaderOrFooterWidth($header-left-width,
                                        $sequenceName, 'odd')"/>
    <xsl:variable name="hcoW" 
      select="h:evalHeaderOrFooterWidth($header-center-width,
                                        $sequenceName, 'odd')"/>
    <xsl:variable name="hroW" 
      select="h:evalHeaderOrFooterWidth($header-right-width,
                                        $sequenceName, 'odd')"/>

    <xsl:if test="exists($hlo) or exists($hco) or exists($hro)">
      <fo:static-content flow-name="odd-page-header">
        <xsl:call-template name="tabularHeader">
          <xsl:with-param name="left" select="$hlo"/>
          <xsl:with-param name="center" select="$hco"/>
          <xsl:with-param name="right" select="$hro"/>
          <xsl:with-param name="isFooter" select="false()"/>
          <xsl:with-param name="leftWidth" select="$hloW"/>
          <xsl:with-param name="centerWidth" select="$hcoW"/>
          <xsl:with-param name="rightWidth" select="$hroW"/>
        </xsl:call-template>
      </fo:static-content>
    </xsl:if>

    <!-- For a single-sided doc, first-XXX and even-XXX flows don't exist -->
    <xsl:if test="$two-sided eq 'yes'">
      <xsl:variable name="hle"
        select="h:evalHeaderOrFooter($header-left,
                                     $sequenceName, 'even', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>
      <xsl:variable name="hce" 
        select="h:evalHeaderOrFooter($header-center,
                                     $sequenceName, 'even', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>
      <xsl:variable name="hre" 
        select="h:evalHeaderOrFooter($header-right,
                                     $sequenceName, 'even', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>

      <xsl:variable name="hleW"
        select="h:evalHeaderOrFooterWidth($header-left-width,
                                          $sequenceName, 'even')"/>
      <xsl:variable name="hceW" 
        select="h:evalHeaderOrFooterWidth($header-center-width,
                                          $sequenceName, 'even')"/>
      <xsl:variable name="hreW" 
        select="h:evalHeaderOrFooterWidth($header-right-width,
                                          $sequenceName, 'even')"/>

      <xsl:if test="exists($hle) or exists($hce) or exists($hre)">
        <fo:static-content flow-name="even-page-header">
          <xsl:call-template name="tabularHeader">
            <xsl:with-param name="left" select="$hle"/>
            <xsl:with-param name="center" select="$hce"/>
            <xsl:with-param name="right" select="$hre"/>
            <xsl:with-param name="isFooter" select="false()"/>
            <xsl:with-param name="leftWidth" select="$hleW"/>
            <xsl:with-param name="centerWidth" select="$hceW"/>
            <xsl:with-param name="rightWidth" select="$hreW"/>
          </xsl:call-template>
        </fo:static-content>
      </xsl:if>

      <xsl:variable name="fl1" 
        select="h:evalHeaderOrFooter($footer-left,
                                     $sequenceName, 'first', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>
      <xsl:variable name="fc1" 
        select="h:evalHeaderOrFooter($footer-center,
                                     $sequenceName, 'first', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>
      <xsl:variable name="fr1" 
        select="h:evalHeaderOrFooter($footer-right,
                                     $sequenceName, 'first', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>

      <xsl:variable name="fl1W" 
        select="h:evalHeaderOrFooterWidth($footer-left-width,
                                          $sequenceName, 'first')"/>
      <xsl:variable name="fc1W" 
        select="h:evalHeaderOrFooterWidth($footer-center-width,
                                          $sequenceName, 'first')"/>
      <xsl:variable name="fr1W" 
        select="h:evalHeaderOrFooterWidth($footer-right-width,
                                          $sequenceName, 'first')"/>

      <xsl:if test="exists($fl1) or exists($fc1) or exists($fr1)">
        <fo:static-content flow-name="first-page-footer">
          <xsl:call-template name="tabularHeader">
            <xsl:with-param name="left" select="$fl1"/>
            <xsl:with-param name="center" select="$fc1"/>
            <xsl:with-param name="right" select="$fr1"/>
            <xsl:with-param name="isFooter" select="true()"/>
            <xsl:with-param name="leftWidth" select="$fl1W"/>
            <xsl:with-param name="centerWidth" select="$fc1W"/>
            <xsl:with-param name="rightWidth" select="$fr1W"/>
          </xsl:call-template>
        </fo:static-content>
      </xsl:if>
    </xsl:if>

    <xsl:variable name="flo" 
      select="h:evalHeaderOrFooter($footer-left,
                                   $sequenceName, 'odd', 
                                   $documentTitle, $documentDate,
                                   $chapterTitle, $section1Title, $divTitle,
                                   $pageNumber, $pageCount,
                                   $lineBreak)"/>
    <xsl:variable name="fco" 
      select="h:evalHeaderOrFooter($footer-center,
                                   $sequenceName, 'odd', 
                                   $documentTitle, $documentDate,
                                   $chapterTitle, $section1Title, $divTitle,
                                   $pageNumber, $pageCount,
                                   $lineBreak)"/>
    <xsl:variable name="fro" 
      select="h:evalHeaderOrFooter($footer-right,
                                   $sequenceName, 'odd', 
                                   $documentTitle, $documentDate,
                                   $chapterTitle, $section1Title, $divTitle,
                                   $pageNumber, $pageCount,
                                   $lineBreak)"/>

    <xsl:variable name="floW" 
      select="h:evalHeaderOrFooterWidth($footer-left-width,
                                        $sequenceName, 'odd')"/>
    <xsl:variable name="fcoW" 
      select="h:evalHeaderOrFooterWidth($footer-center-width,
                                        $sequenceName, 'odd')"/>
    <xsl:variable name="froW" 
      select="h:evalHeaderOrFooterWidth($footer-right-width,
                                        $sequenceName, 'odd')"/>

    <xsl:if test="exists($flo) or exists($fco) or exists($fro)">
      <fo:static-content flow-name="odd-page-footer">
        <xsl:call-template name="tabularHeader">
          <xsl:with-param name="left" select="$flo"/>
          <xsl:with-param name="center" select="$fco"/>
          <xsl:with-param name="right" select="$fro"/>
          <xsl:with-param name="isFooter" select="true()"/>
          <xsl:with-param name="leftWidth" select="$floW"/>
          <xsl:with-param name="centerWidth" select="$fcoW"/>
          <xsl:with-param name="rightWidth" select="$froW"/>
        </xsl:call-template>
      </fo:static-content>
    </xsl:if>

    <!-- For a single-sided doc, first-XXX and even-XXX flows don't exist -->
    <xsl:if test="$two-sided eq 'yes'">
      <xsl:variable name="fle" 
        select="h:evalHeaderOrFooter($footer-left, 
                                     $sequenceName, 'even', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>
      <xsl:variable name="fce" 
        select="h:evalHeaderOrFooter($footer-center, 
                                     $sequenceName, 'even', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>
      <xsl:variable name="fre" 
        select="h:evalHeaderOrFooter($footer-right, 
                                     $sequenceName, 'even', 
                                     $documentTitle, $documentDate,
                                     $chapterTitle, $section1Title, $divTitle,
                                     $pageNumber, $pageCount,
                                     $lineBreak)"/>

      <xsl:variable name="fleW" 
        select="h:evalHeaderOrFooterWidth($footer-left-width, 
                                          $sequenceName, 'even')"/>
      <xsl:variable name="fceW" 
        select="h:evalHeaderOrFooterWidth($footer-center-width, 
                                          $sequenceName, 'even')"/>
      <xsl:variable name="freW" 
        select="h:evalHeaderOrFooterWidth($footer-right-width, 
                                          $sequenceName, 'even')"/>

      <xsl:if test="exists($fle) or exists($fce) or exists($fre)">
        <fo:static-content flow-name="even-page-footer">
          <xsl:call-template name="tabularHeader">
            <xsl:with-param name="left" select="$fle"/>
            <xsl:with-param name="center" select="$fce"/>
            <xsl:with-param name="right" select="$fre"/>
            <xsl:with-param name="isFooter" select="true()"/>
            <xsl:with-param name="leftWidth" select="$fleW"/>
            <xsl:with-param name="centerWidth" select="$fceW"/>
            <xsl:with-param name="rightWidth" select="$freW"/>
          </xsl:call-template>
        </fo:static-content>
      </xsl:if>
    </xsl:if>

  </xsl:template>

  <xsl:function name="h:evalHeaderOrFooterWidth" as="xs:string">
    <xsl:param name="spec" as="xs:string"/>

    <xsl:param name="sequenceName" as="xs:string"/>
    <xsl:param name="firstOrEvenOrOdd" as="xs:string"/>

    <xsl:variable name="spec2" select="normalize-space($spec)"/>
    <xsl:choose>
      <xsl:when test="number($spec2) ge 1">
        <!-- Simple, most common case: no conditions (e.g. "2"). -->
        <xsl:sequence select="$spec2"/>
      </xsl:when>

      <xsl:otherwise>
        <xsl:variable name="filter" 
          select="h:createFilter($sequenceName, $firstOrEvenOrOdd)"/>

        <xsl:variable name="valueSpecs"
                      select="for $case in tokenize($spec2, ';;')
                              return
                                  h:testCase($case, $filter)"/>
        <xsl:variable name="valueSpec" select="$valueSpecs[1]"/>

        <xsl:choose>
          <xsl:when test="number($valueSpec) ge 1">
            <xsl:sequence select="$valueSpec"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:message terminate="yes" 
              select="concat('cannot evaluate &quot;', $spec2, 
                             '&quot; as a proportional column width')"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xsl:function name="h:evalHeaderOrFooter" as="item()*">
    <xsl:param name="spec" as="xs:string"/>

    <xsl:param name="sequenceName" as="xs:string"/>
    <xsl:param name="firstOrEvenOrOdd" as="xs:string"/>

    <xsl:param name="documentTitle" as="xs:string?"/>
    <xsl:param name="documentDate" as="xs:string?"/>

    <xsl:param name="chapterTitle" as="item()*"/>
    <xsl:param name="section1Title" as="item()*"/>
    <xsl:param name="divTitle" as="item()*"/>

    <xsl:param name="pageNumber" as="item()*"/>
    <xsl:param name="pageCount" as="item()*"/>

    <xsl:param name="lineBreak" as="item()*"/>

    <xsl:variable name="filter" 
                  select="h:createFilter($sequenceName, $firstOrEvenOrOdd)"/>

    <xsl:variable name="valueSpecs"
                  select="for $case in tokenize($spec, ';;')
                          return
                              h:testCase($case, $filter)"/>
    <xsl:variable name="valueSpec" select="$valueSpecs[1]"/>

    <xsl:choose>
      <xsl:when test="$valueSpec ne ''">
        <!-- The Private Use Area of the Unicode Basic Multilingual Plane is 
             E000 to F8FF. 
             We'll use:
             E000=value separator
             E001=page-sequence
             E002=document-title
             E003=document-date
             E004=chapter-title
             E005=section1-title
             E006=division-title
             E007=page-number
             E008=page-count
             E009=break
             E00A=image() -->

        <xsl:variable name="valueSpec1" 
          select="if (contains($valueSpec, '{{page-sequence}}'))
                  then replace($valueSpec, '\{\{page-sequence\}\}', 
                               '&#xE000;&#xE001;&#xE000;')
                  else $valueSpec"/>
        <xsl:variable name="valueSpec2" 
          select="if (contains($valueSpec1, '{{document-title}}'))
                  then replace($valueSpec1, '\{\{document-title\}\}', 
                               '&#xE000;&#xE002;&#xE000;')
                  else $valueSpec1"/>
        <xsl:variable name="valueSpec3" 
          select="if (contains($valueSpec2, '{{document-date}}'))
                  then replace($valueSpec2, '\{\{document-date\}\}', 
                               '&#xE000;&#xE003;&#xE000;')
                  else $valueSpec2"/>
        <xsl:variable name="valueSpec4" 
          select="if (contains($valueSpec3, '{{chapter-title}}'))
                  then replace($valueSpec3, '\{\{chapter-title\}\}', 
                               '&#xE000;&#xE004;&#xE000;')
                  else $valueSpec3"/>
        <xsl:variable name="valueSpec5" 
          select="if (contains($valueSpec4, '{{section1-title}}'))
                  then replace($valueSpec4, '\{\{section1-title\}\}', 
                               '&#xE000;&#xE005;&#xE000;')
                  else $valueSpec4"/>
        <xsl:variable name="valueSpec6" 
          select="if (contains($valueSpec5, '{{division-title}}'))
                  then replace($valueSpec5, '\{\{division-title\}\}', 
                               '&#xE000;&#xE006;&#xE000;')
                  else $valueSpec5"/>
        <xsl:variable name="valueSpec7" 
          select="if (contains($valueSpec6, '{{page-number}}'))
                  then replace($valueSpec6, '\{\{page-number\}\}', 
                               '&#xE000;&#xE007;&#xE000;')
                  else $valueSpec6"/>
        <xsl:variable name="valueSpec8" 
          select="if (contains($valueSpec7, '{{page-count}}'))
                  then replace($valueSpec7, '\{\{page-count\}\}', 
                               '&#xE000;&#xE008;&#xE000;')
                  else $valueSpec7"/>
        <xsl:variable name="valueSpec9" 
          select="if (contains($valueSpec8, '{{break}}'))
                  then replace($valueSpec8, '\{\{break\}\}', 
                               '&#xE000;&#xE009;&#xE000;')
                  else $valueSpec8"/>
        <!-- Beware that the image path may contain parenthesis. -->
        <xsl:variable name="valueSpecA" 
          select="if (contains($valueSpec9, '{{image(' ))
                  then replace($valueSpec9, '\{\{image\((.+)\)\}\}', 
                               '&#xE000;&#xE00A;$1&#xE000;')
                  else $valueSpec9"/>

        <xsl:sequence 
          select="for $t in tokenize($valueSpecA, '&#xE000;')
                  return
                    if ($t eq '&#xE001;') then $sequenceName
                    else 
                      if ($t eq '&#xE002;') then $documentTitle
                      else 
                        if ($t eq '&#xE003;') then $documentDate
                        else 
                          if ($t eq '&#xE004;') then $chapterTitle
                          else 
                            if ($t eq '&#xE005;') then $section1Title
                            else 
                              if ($t eq '&#xE006;') then $divTitle
                              else 
                                if ($t eq '&#xE007;') then $pageNumber
                                else 
                                  if ($t eq '&#xE008;') then $pageCount
                                  else 
                                    if ($t eq '&#xE009;') then $lineBreak
                                    else 
                                      if (starts-with($t, '&#xE00A;')) 
                                      then h:toExternalGraphic(substring($t, 2))
                                      else
                                        if ($t ne '') then $t else ()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xsl:function name="h:testCase" as="xs:string?">
    <xsl:param name="case" as="xs:string"/>
    <xsl:param name="filter" as="xs:string+"/>

    <xsl:variable name="conditions" select="tokenize($case, '::')"/>
    <xsl:variable name="conditionCount" select="count($conditions)"/>

    <xsl:choose>
      <xsl:when test="$conditionCount eq 0">
        <!-- Empty case (e.g. "foo:: bar;; ;;"). Skip it. -->
        <xsl:sequence select="()"/>
      </xsl:when>

      <xsl:when test="$conditionCount eq 1">
        <!-- Simple case having no condition (e.g. "bar"). -->
        <xsl:sequence select="normalize-space($case)"/>
      </xsl:when>

      <xsl:otherwise>
        <!-- May be the empty string (e.g. "foo:: ;;"). -->
        <xsl:variable name="valueSpec" 
                      select="normalize-space($conditions[last()])"/>

        <xsl:variable name="matches"
          select="for $cond in subsequence($conditions, 1, $conditionCount - 1)
                  return
                      if (h:testCondition($cond, $filter))
                      then true()
                      else ()"/>

        <xsl:sequence select="if (exists($matches)) then $valueSpec else ()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xsl:function name="h:createFilter" as="xs:string+">
    <xsl:param name="sequenceName" as="xs:string"/>
    <xsl:param name="firstOrEvenOrOdd" as="xs:string"/>

    <!-- Example: ('one-side', 'section1', 'odd') -->
    <xsl:sequence select="(if ($two-sided eq 'yes') 
                           then 'two-sides' 
                           else 'one-side', 
                           $sequenceName, 
                           $firstOrEvenOrOdd)"/>
  </xsl:function>

  <xsl:function name="h:testCondition" as="xs:boolean">
    <xsl:param name="condition" as="xs:string"/>
    <xsl:param name="filter" as="xs:string+"/>

    <xsl:variable name="condition2" select="normalize-space($condition)"/>
    <xsl:choose>
      <xsl:when test="$condition2 eq ''">
        <!-- Empty condition means no condition (e.g. ":: bar;;"). -->
        <xsl:sequence select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="simpleConditions" 
                      select="tokenize($condition2, '\s+')"/>

        <xsl:variable name="matches"
          select="for $simpleCondition in $simpleConditions
                  return
                      if (h:testSimpleCondition($simpleCondition, $filter))
                      then true()
                      else ()"/>

        <xsl:sequence select="count($matches) eq count($simpleConditions)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xsl:function name="h:testSimpleCondition" as="xs:boolean">
    <xsl:param name="simpleCondition" as="xs:string"/>
    <xsl:param name="filter" as="xs:string+"/>

    <!-- Note that whitespace before and after "||" is not supported. -->
    <!-- Skip an empty alternative. -->
    <xsl:variable name="matches"
      select="for $alternative in tokenize($simpleCondition, '\|\|')
              return
                  if ($alternative ne '' and 
                      exists(index-of($filter, $alternative)))
                  then true()
                  else ()"/>

    <xsl:sequence select="exists($matches)"/>
  </xsl:function>

  <xsl:function name="h:toExternalGraphic" as="element()">
    <xsl:param name="uri" as="xs:string"/>

    <fo:external-graphic
      src="{concat('url(', resolve-uri($uri, URI:userDirectory()), ')')}"/>
  </xsl:function>

  <!-- tabularHeader ========== -->

  <xsl:attribute-set name="header-or-footer">
    <xsl:attribute name="hyphenate">false</xsl:attribute>
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
    <xsl:attribute name="font-size">0.7em</xsl:attribute>
    <!-- Not $base-line-height div 0.7 -->
    <xsl:attribute name="line-height" select="$base-line-height"/>
    <xsl:attribute name="color">#808080</xsl:attribute>
    <xsl:attribute name="border-color">#808080</xsl:attribute>
    <xsl:attribute name="border-width">0.25pt</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="header" use-attribute-sets="header-or-footer"/>

  <xsl:attribute-set name="footer" use-attribute-sets="header-or-footer"/>

  <xsl:attribute-set name="header-or-footer-left">
    <xsl:attribute name="text-align">left</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="header-or-footer-center">
    <xsl:attribute name="text-align">center</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="header-or-footer-right">
    <xsl:attribute name="text-align">right</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="header-left" 
                     use-attribute-sets="header-or-footer-left"/>
  <xsl:attribute-set name="header-center" 
                     use-attribute-sets="header-or-footer-center"/>
  <xsl:attribute-set name="header-right" 
                     use-attribute-sets="header-or-footer-right"/>

  <xsl:attribute-set name="footer-left" 
                     use-attribute-sets="header-or-footer-left"/>
  <xsl:attribute-set name="footer-center" 
                     use-attribute-sets="header-or-footer-center"/>
  <xsl:attribute-set name="footer-right" 
                     use-attribute-sets="header-or-footer-right"/>

  <xsl:template name="tabularHeader">
    <xsl:param name="left" select="()"/>
    <xsl:param name="center" select="()"/>
    <xsl:param name="right" select="()"/>
    <xsl:param name="isFooter" select="false()"/>
    <xsl:param name="leftWidth" select="'2'"/>
    <xsl:param name="centerWidth" select="'6'"/>
    <xsl:param name="rightWidth" select="'2'"/>

    <xsl:choose>
      <xsl:when test="$isFooter">
        <fo:block xsl:use-attribute-sets="footer">
          <xsl:if test="$footer-separator eq 'yes'">
            <xsl:attribute name="border-top-style">solid</xsl:attribute>
            <xsl:attribute name="padding-top">0.165em</xsl:attribute>
          </xsl:if>

          <fo:table table-layout="fixed" width="100%">
            <fo:table-column column-number="1" 
                             column-width="{concat('proportional-column-width(',
                                                   $leftWidth, ')')}"/>
            <fo:table-column column-number="2" 
                             column-width="{concat('proportional-column-width(',
                                                   $centerWidth, ')')}"/>
            <fo:table-column column-number="3" 
                             column-width="{concat('proportional-column-width(',
                                                   $rightWidth, ')')}"/>
            <fo:table-body>
              <fo:table-row>
                <fo:table-cell start-indent="0" display-align="before"
                               xsl:use-attribute-sets="footer-left">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($left)">
                        <xsl:copy-of select="$left"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell start-indent="0" display-align="before"
                               xsl:use-attribute-sets="footer-center">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($center)">
                        <xsl:copy-of select="$center"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell start-indent="0" display-align="before"
                               xsl:use-attribute-sets="footer-right">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($right)">
                        <xsl:copy-of select="$right"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
              </fo:table-row>
            </fo:table-body>
          </fo:table>
        </fo:block>
      </xsl:when>

      <xsl:otherwise>
        <fo:block xsl:use-attribute-sets="header">
          <xsl:if test="$header-separator eq 'yes'">
            <xsl:attribute name="padding-bottom">0.165em</xsl:attribute>
            <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
          </xsl:if>

          <fo:table table-layout="fixed" width="100%">
            <fo:table-column column-number="1" 
                             column-width="{concat('proportional-column-width(',
                                                   $leftWidth, ')')}"/>
            <fo:table-column column-number="2" 
                             column-width="{concat('proportional-column-width(',
                                                   $centerWidth, ')')}"/>
            <fo:table-column column-number="3" 
                             column-width="{concat('proportional-column-width(',
                                                   $rightWidth, ')')}"/>
            <fo:table-body>
              <fo:table-row>
                <fo:table-cell start-indent="0" display-align="after"
                               xsl:use-attribute-sets="header-left">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($left)">
                        <xsl:copy-of select="$left"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell start-indent="0" display-align="after"
                               xsl:use-attribute-sets="header-center">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($center)">
                        <xsl:copy-of select="$center"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell start-indent="0" display-align="after"
                               xsl:use-attribute-sets="header-right">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($right)">
                        <xsl:copy-of select="$right"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
              </fo:table-row>
            </fo:table-body>
          </fo:table>
        </fo:block>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
