<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2017-2019 XMLmind Software. All rights reserved.
|
| Author: Hussein Shafie (hussein@xmlmind.com)
|
| This file is part of the XMLmind Ebook Compiler project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                exclude-result-prefixes="xs"
                version="2.0">

  <xsl:param name="apply-css-styles" select="'yes'"/>

  <xsl:param name="base-font-size" select="'10pt'"/>
  <xsl:param name="base-line-height" select="'1.4'"/>
  <xsl:param name="font-family" select="'serif'"/>
  <xsl:param name="justified" select="'no'"/>
  <xsl:param name="hyphenate" select="'no'"/>

  <!-- Few glyphs are supported by the combination of XSL-FO
       processor/built-in PDF fonts. These are supported by FOP and
       XEP. -->
  <xsl:param name="ul-li-bullets" select="'&#x2022; &#x2013;'"/>
  <xsl:param name="ulLiBullets" select="tokenize($ul-li-bullets, '\s+')" />

  <xsl:param name="show-external-links" select="'no'"/>
  <xsl:param name="external-href-before" select="' ['"/>
  <xsl:param name="external-href-after" select="']'"/>

  <xsl:param name="show-xref-page" select="'no'"/>
  <xsl:param name="page-ref-before" select="' ['"/>
  <xsl:param name="page-ref-after" select="']'"/>

  <xsl:param name="show-map-links" select="'yes'"/>

  <xsl:param name="use-note-label" select="'no'"/>
  <xsl:param name="use-note-icon" select="'no'"/>
  <!-- A valid dimension must be specified here.
       This dimension must have a unit. 
       (32px at 96DPI gives 0.333in) -->
  <xsl:param name="note-icon-width" select="'0.333in'"/>
  <xsl:param name="note-icon-height" select="'0.333in'"/>

  <!-- Workaround an XFC limitation.
       Other possible values are: nav, header, footer. -->
  <xsl:param name="xfc-render-as-table" select="'admonition aside blockquote'"/>
  <xsl:param name="xfcRenderAsTable"
             select="tokenize($xfc-render-as-table, '\s+')"/>

  <xsl:param name="pdf-outline" select="'no'"/>

  <!-- URI of an image file. If the URI is relative, it is relative 
       to the current working directory. -->
  <xsl:param name="watermark-image" select="''" />

  <!-- One or more of: 'blank', 'title', 'toc', 'booklist', 'frontmatter', 
       'body', 'backmatter', 'index', 'all'. -->
  <xsl:param name="watermark" select="'all'"/>
  <xsl:variable name="watermarkList" select="tokenize($watermark, '\s+')"/>

  <!-- Page dimension and orientation ==================================== -->

  <xsl:param name="paper-type" select="'A4'"/>

  <xsl:param name="page-orientation" select="'portrait'"/>

  <xsl:param name="page-width"
             select="if ($page-orientation eq 'portrait') 
                     then $paper-width
                     else $paper-height"/>
  <xsl:param name="page-height" 
             select="if ($page-orientation eq 'portrait') 
                     then $paper-height 
                     else $paper-width"/>

  <xsl:param name="paper-width" as="xs:string">
    <xsl:variable name="paperType" select="lower-case($paper-type)"/>
    <xsl:choose>
      <xsl:when test="$paperType eq 'letter'">8.5in</xsl:when>
      <xsl:when test="$paperType eq 'legal'">8.5in</xsl:when>
      <xsl:when test="$paperType eq 'ledger'">17in</xsl:when>
      <xsl:when test="$paperType eq 'tabloid'">11in</xsl:when>
      <xsl:when test="$paperType eq 'a0'">841mm</xsl:when>
      <xsl:when test="$paperType eq 'a1'">594mm</xsl:when>
      <xsl:when test="$paperType eq 'a2'">420mm</xsl:when>
      <xsl:when test="$paperType eq 'a3'">297mm</xsl:when>
      <xsl:when test="$paperType eq 'a4'">210mm</xsl:when>
      <xsl:when test="$paperType eq 'a5'">148mm</xsl:when>
      <xsl:when test="$paperType eq 'a6'">105mm</xsl:when>
      <xsl:when test="$paperType eq 'a7'">74mm</xsl:when>
      <xsl:when test="$paperType eq 'a8'">52mm</xsl:when>
      <xsl:when test="$paperType eq 'a9'">37mm</xsl:when>
      <xsl:when test="$paperType eq 'a10'">26mm</xsl:when>
      <xsl:when test="$paperType eq 'b0'">1000mm</xsl:when>
      <xsl:when test="$paperType eq 'b1'">707mm</xsl:when>
      <xsl:when test="$paperType eq 'b2'">500mm</xsl:when>
      <xsl:when test="$paperType eq 'b3'">353mm</xsl:when>
      <xsl:when test="$paperType eq 'b4'">250mm</xsl:when>
      <xsl:when test="$paperType eq 'b5'">176mm</xsl:when>
      <xsl:when test="$paperType eq 'b6'">125mm</xsl:when>
      <xsl:when test="$paperType eq 'b7'">88mm</xsl:when>
      <xsl:when test="$paperType eq 'b8'">62mm</xsl:when>
      <xsl:when test="$paperType eq 'b9'">44mm</xsl:when>
      <xsl:when test="$paperType eq 'b10'">31mm</xsl:when>
      <xsl:when test="$paperType eq 'c0'">917mm</xsl:when>
      <xsl:when test="$paperType eq 'c1'">648mm</xsl:when>
      <xsl:when test="$paperType eq 'c2'">458mm</xsl:when>
      <xsl:when test="$paperType eq 'c3'">324mm</xsl:when>
      <xsl:when test="$paperType eq 'c4'">229mm</xsl:when>
      <xsl:when test="$paperType eq 'c5'">162mm</xsl:when>
      <xsl:when test="$paperType eq 'c6'">114mm</xsl:when>
      <xsl:when test="$paperType eq 'c7'">81mm</xsl:when>
      <xsl:when test="$paperType eq 'c8'">57mm</xsl:when>
      <xsl:when test="$paperType eq 'c9'">40mm</xsl:when>
      <xsl:when test="$paperType eq 'c10'">28mm</xsl:when>
      <xsl:otherwise>210mm</xsl:otherwise>
    </xsl:choose>
  </xsl:param>

  <xsl:param name="paper-height" as="xs:string">
    <xsl:variable name="paperType" select="lower-case($paper-type)"/>
    <xsl:choose>
      <xsl:when test="$paperType eq 'letter'">11in</xsl:when>
      <xsl:when test="$paperType eq 'legal'">14in</xsl:when>
      <xsl:when test="$paperType eq 'ledger'">11in</xsl:when>
      <xsl:when test="$paperType eq 'tabloid'">17in</xsl:when>
      <xsl:when test="$paperType eq 'a0'">1189mm</xsl:when>
      <xsl:when test="$paperType eq 'a1'">841mm</xsl:when>
      <xsl:when test="$paperType eq 'a2'">594mm</xsl:when>
      <xsl:when test="$paperType eq 'a3'">420mm</xsl:when>
      <xsl:when test="$paperType eq 'a4'">297mm</xsl:when>
      <xsl:when test="$paperType eq 'a5'">210mm</xsl:when>
      <xsl:when test="$paperType eq 'a6'">148mm</xsl:when>
      <xsl:when test="$paperType eq 'a7'">105mm</xsl:when>
      <xsl:when test="$paperType eq 'a8'">74mm</xsl:when>
      <xsl:when test="$paperType eq 'a9'">52mm</xsl:when>
      <xsl:when test="$paperType eq 'a10'">37mm</xsl:when>
      <xsl:when test="$paperType eq 'b0'">1414mm</xsl:when>
      <xsl:when test="$paperType eq 'b1'">1000mm</xsl:when>
      <xsl:when test="$paperType eq 'b2'">707mm</xsl:when>
      <xsl:when test="$paperType eq 'b3'">500mm</xsl:when>
      <xsl:when test="$paperType eq 'b4'">353mm</xsl:when>
      <xsl:when test="$paperType eq 'b5'">250mm</xsl:when>
      <xsl:when test="$paperType eq 'b6'">176mm</xsl:when>
      <xsl:when test="$paperType eq 'b7'">125mm</xsl:when>
      <xsl:when test="$paperType eq 'b8'">88mm</xsl:when>
      <xsl:when test="$paperType eq 'b9'">62mm</xsl:when>
      <xsl:when test="$paperType eq 'b10'">44mm</xsl:when>
      <xsl:when test="$paperType eq 'c0'">1297mm</xsl:when>
      <xsl:when test="$paperType eq 'c1'">917mm</xsl:when>
      <xsl:when test="$paperType eq 'c2'">648mm</xsl:when>
      <xsl:when test="$paperType eq 'c3'">458mm</xsl:when>
      <xsl:when test="$paperType eq 'c4'">324mm</xsl:when>
      <xsl:when test="$paperType eq 'c5'">229mm</xsl:when>
      <xsl:when test="$paperType eq 'c6'">162mm</xsl:when>
      <xsl:when test="$paperType eq 'c7'">114mm</xsl:when>
      <xsl:when test="$paperType eq 'c8'">81mm</xsl:when>
      <xsl:when test="$paperType eq 'c9'">57mm</xsl:when>
      <xsl:when test="$paperType eq 'c10'">40mm</xsl:when>
      <xsl:otherwise>297mm</xsl:otherwise>
    </xsl:choose>
  </xsl:param>

  <xsl:param name="two-sided" select="'no'"/>

  <xsl:param name="page-top-margin" select="'0.5in'"/>
  <xsl:param name="page-bottom-margin" select="'0.5in'"/>
  <xsl:param name="page-inner-margin"
             select="if ($two-sided eq 'yes') then '1.25in' else '1in'"/>
  <xsl:param name="page-outer-margin"
             select="if ($two-sided eq 'yes') then '0.75in' else '1in'"/>

  <xsl:param name="body-top-margin" select="'0.5in'"/>
  <xsl:param name="body-bottom-margin" select="'0.5in'"/>

  <xsl:param name="header-height" select="'0.4in'"/>
  <xsl:param name="footer-height" select="'0.4in'"/>

  <xsl:param name="index-column-count" select="2"/>
  <xsl:param name="index-column-gap" select="'2em'"/>

  <!-- Headers and footers =============================================== -->

  <xsl:param name="header-left" select="''"/>
  <xsl:param name="header-center" select="'{{document-title}}'"/>
  <xsl:param name="header-right" select="''"/>

  <xsl:param name="footer-left">
    two-sides even:: {{page-number}}
  </xsl:param>

  <xsl:param name="footer-right">
    two-sides first||odd:: {{page-number}};;
    one-side:: {{page-number}}
  </xsl:param>

  <!-- ====================
       About footer-center:

       * One-sided document:

         part, chapter, appendices or appendix title.

       * Two-sided document:

         First page: nothing.
         Even page: part, chapter, appendices or appendix title 
         Odd page: running ``chapter title'' which retrieves
                   part, chapter, appendices, appendix or 
                   part|chapter|appendix/section1 title.

       Note that chapter-title is non-empty only inside part, chapter,
       appendices or appendix (therefore no need to specify "body").  
       ==================== -->

  <xsl:param name="footer-center">
    two-sides even:: {{chapter-title}};;
    two-sides body odd:: {{section1-title}};;
    one-side:: {{chapter-title}}
  </xsl:param>

  <xsl:param name="header-separator" select="'yes'"/>
  <xsl:param name="footer-separator" select="'yes'"/>

  <xsl:param name="header-left-width" select="'2'"/>
  <xsl:param name="header-center-width" select="'6'"/>
  <xsl:param name="header-right-width" select="'2'"/>

  <xsl:param name="footer-left-width"  select="'2'"/>
  <xsl:param name="footer-center-width" select="'6'"/>
  <xsl:param name="footer-right-width" select="'2'"/>

  <!-- Advanced parameters =============================================== -->

  <!-- Prepended to the src attribute of the generated fo:external-graphic,
       unless the src of the XHTML img is absolute. 
       This value must be empty or must end with a '/'. -->
  <xsl:param name="img-src-path" select="''"/>

  <xsl:param name="resolve-img-src" select="'no'"/>

  <xsl:param name="resolve-a-href" select="'no'"/>

  <!-- Screen resolution in DPI. Used to convert px to pt. -->
  <xsl:param name="screen-resolution" select="96.0" />

  <!-- System parameters ================================================= -->

  <xsl:param name="xsl-resources-directory" 
             select="resolve-uri('resources/', document-uri(doc('')))"/>
  <xsl:variable name="xslResourcesDir" 
                select="if (ends-with($xsl-resources-directory, '/')) 
                        then $xsl-resources-directory
                        else concat($xsl-resources-directory, '/')"/>

  <!-- Parameters set by the client code, not by the end user ============ -->

  <xsl:param name="foProcessor" select="''"/>

</xsl:stylesheet>
