<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2017 XMLmind Software. All rights reserved.
|
| Author: Hussein Shafie (hussein@xmlmind.com)
|
| This file is part of the XMLmind Ebook Compiler project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:html="http://www.w3.org/1999/xhtml"
                xmlns:rx="http://www.renderx.com/XSL/Extensions"
                xmlns:u="http://www.xmlmind.com/namespace/ebookc"
                exclude-result-prefixes="xs html u"
                version="2.0">

  <!-- xsl11PDFOutline =================================================== -->
  <!-- Standard XSL 1.1 facility supported by FOP and AntennaHouse. -->

  <xsl:template name="xsl11PDFOutline">
    <fo:bookmark-tree>
      <!-- Current context is html:section[u:has(@class, 'role-toc')]. -->
      <fo:bookmark internal-destination="{@id}" starting-state="hide">
        <fo:bookmark-title>
          <xsl:value-of select="html:h1[u:has(@class, 'role-toc-title')]"/>
        </fo:bookmark-title>
      </fo:bookmark>

      <xsl:apply-templates select="html:ul[u:has(@class, 'role-booklist-entries')]" 
                           mode="xsl11-pdf-outline"/>
    </fo:bookmark-tree>
  </xsl:template>

  <xsl:template match="html:ul[u:has(@class, 'role-booklist-entries')]"
                mode="xsl11-pdf-outline">
    <xsl:apply-templates mode="xsl11-pdf-outline"/>
  </xsl:template>

  <xsl:template match="html:li[u:has(@class, 'role-booklist-entry')]"
                mode="xsl11-pdf-outline">
    <xsl:variable name="link" 
                  select="./html:a[u:matches(@class, 'role-', '-ref')]"/>
    <xsl:variable name="ref" select="substring-after($link[1]/@href, '#')"/>

    <xsl:if test="$ref ne ''">
      <!-- starting-state=show|hide. Default: show -->
      <fo:bookmark internal-destination="{$ref}" starting-state="hide">
        <xsl:apply-templates mode="xsl11-pdf-outline"/>
      </fo:bookmark>
    </xsl:if>
  </xsl:template>

  <xsl:template match="html:a[u:matches(@class, 'role-', '-ref')]"
                mode="xsl11-pdf-outline">
    <fo:bookmark-title>
      <xsl:value-of select="."/>
    </fo:bookmark-title>
  </xsl:template>

  <!-- xepPDFOutline ===================================================== -->

  <xsl:template name="xepPDFOutline">
    <rx:outline>
      <!-- Current context is html:section[u:has(@class, 'role-toc')]. -->
      <rx:bookmark internal-destination="{@id}" collapse-subtree="true">
        <rx:bookmark-label>
          <xsl:value-of select="html:h1[u:has(@class, 'role-toc-title')]"/>
        </rx:bookmark-label>
      </rx:bookmark>

      <xsl:apply-templates select="html:ul[u:has(@class, 'role-booklist-entries')]" 
                           mode="xep-pdf-outline"/>
    </rx:outline>
  </xsl:template>

  <xsl:template match="html:ul[u:has(@class, 'role-booklist-entries')]"
                mode="xep-pdf-outline">
    <xsl:apply-templates mode="xep-pdf-outline"/>
  </xsl:template>

  <xsl:template match="html:li[u:has(@class, 'role-booklist-entry')]"
                mode="xep-pdf-outline">
    <xsl:variable name="link" 
                  select="./html:a[u:matches(@class, 'role-', '-ref')]"/>
    <xsl:variable name="ref" select="substring-after($link[1]/@href, '#')"/>

    <xsl:if test="$ref ne ''">
      <!-- collapse-subtree=false|true -->
      <rx:bookmark internal-destination="{$ref}" collapse-subtree="true">
        <xsl:apply-templates mode="xep-pdf-outline"/>
      </rx:bookmark>
    </xsl:if>
  </xsl:template>

  <xsl:template match="html:a[u:matches(@class, 'role-', '-ref')]"
                mode="xep-pdf-outline">
    <rx:bookmark-label>
      <xsl:value-of select="."/>
    </rx:bookmark-label>
  </xsl:template>

</xsl:stylesheet>
