<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2017 XMLmind Software. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind Ebook Compiler project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:whc="http://www.xmlmind.com/whc/schema/whc"
                xmlns="http://www.w3.org/1999/xhtml"
                xmlns:u="http://www.xmlmind.com/namespace/ebookc"
                xpath-default-namespace="http://www.w3.org/1999/xhtml"
                exclude-result-prefixes="u"
                version="2.0">

  <xsl:template name="generateIndex">
    <xsl:result-document href="{resolve-uri($whc-index-basename,base-uri())}"
                         format="xmlOutputFormat">

      <whc:index mergeAndSort="false">
        <xsl:attribute name="termSeparator">
          <xsl:call-template name="localize">
            <xsl:with-param name="message" select="'termSeparator'"/>
          </xsl:call-template>
        </xsl:attribute>

        <xsl:if test="exists(@lang)">
          <xsl:attribute name="xml:lang" select="@lang"/>
        </xsl:if>

        <xsl:apply-templates
          select="//div[u:has(@class, 'role-index-div')]/ul[u:has(@class, 'role-index-entries')]" mode="index"/>
      </whc:index>

    </xsl:result-document>
  </xsl:template>

  <xsl:template match="ul[u:has(@class, 'role-index-entries')]" mode="index">
    <xsl:apply-templates mode="index"/>
  </xsl:template>

  <xsl:template match="li[u:has(@class, 'role-index-entry')]" mode="index">
    <whc:entry>
      <xsl:if test="exists(@id)">
        <xsl:attribute name="xml:id" select="@id"/>
      </xsl:if>

      <!-- Skip text like term and anchor separators. -->
      <xsl:apply-templates select="./*" mode="index"/>
    </whc:entry>
  </xsl:template>

  <xsl:template match="span[u:has(@class, 'role-index-term')]" mode="index">
    <whc:term>
      <xsl:if test="exists(@data-sort-as)">
        <xsl:attribute name="sortAs" select="@data-sort-as"/>
      </xsl:if>

      <xsl:copy-of select="./node()"/>
    </whc:term>
  </xsl:template>

  <xsl:template match="span[u:has(@class, 'role-index-anchor-refs')]" mode="index">
    <xsl:apply-templates select="./*" mode="index"/>
  </xsl:template>

  <xsl:template match="span[u:has(@class, 'role-index-range')]" mode="index">
    <whc:anchor>
      <xsl:copy-of select="a[1]/@href"/>
      <xsl:attribute name="href2" select="string(a[2]/@href)"/>
    </whc:anchor>
  </xsl:template>

  <xsl:template match="a[u:has(@class, 'role-index-anchor-ref')]" mode="index">
    <whc:anchor>
      <xsl:copy-of select="@href"/>
    </whc:anchor>
  </xsl:template>

  <xsl:template match="span[u:has(@class, 'role-index-see')]" mode="index">
    <whc:see>
      <xsl:variable name="link"
                    select="./a[u:has(@class, 'role-index-see-link')]"/>

      <xsl:if test="exists($link)">
        <xsl:attribute name="ref" select="substring-after($link/@href, '#')"/>
        <xsl:copy-of select="$link/node()"/>
      </xsl:if>
    </whc:see>
  </xsl:template>

  <xsl:template match="span[u:has(@class, 'role-index-see-also')]" mode="index">
    <whc:seeAlso>
      <xsl:variable name="link"
                    select="./a[u:has(@class, 'role-index-see-also-link')]"/>

      <xsl:if test="exists($link)">
        <xsl:attribute name="ref" select="substring-after($link/@href, '#')"/>
        <xsl:copy-of select="$link/node()"/>
      </xsl:if>
    </whc:seeAlso>
  </xsl:template>

</xsl:stylesheet>
